# -*- coding: utf-8 -*-
'''
Salt compatibility code
'''
# pylint: disable=import-error,unused-import,invalid-name,W0231,W0233

# Import python libs
from __future__ import absolute_import, unicode_literals, print_function
import sys
import types
import logging

# Import 3rd-party libs
from salt.exceptions import SaltException
from salt.ext.six import binary_type, string_types, text_type, integer_types
from salt.ext.six.moves import cStringIO, StringIO

log = logging.getLogger(__name__)

try:
    # Python >2.5
    import xml.etree.cElementTree as ElementTree
except Exception:
    try:
        # Python >2.5
        import xml.etree.ElementTree as ElementTree
    except Exception:
        try:
            # normal cElementTree install
            import elementtree.cElementTree as ElementTree
        except Exception:
            try:
                # normal ElementTree install
                import elementtree.ElementTree as ElementTree
            except Exception:
                ElementTree = None


# True if we are running on Python 3.
PY3 = sys.version_info.major == 3


if PY3:
    import builtins
    exceptions = builtins
else:
    import exceptions


if ElementTree is not None:
    if not hasattr(ElementTree, 'ParseError'):
        class ParseError(Exception):
            '''
            older versions of ElementTree do not have ParseError
            '''

        ElementTree.ParseError = ParseError


def text_(s, encoding='latin-1', errors='strict'):
    '''
    If ``s`` is an instance of ``binary_type``, return
    ``s.decode(encoding, errors)``, otherwise return ``s``
    '''
    return s.decode(encoding, errors) if isinstance(s, binary_type) else s


def bytes_(s, encoding='latin-1', errors='strict'):
    '''
    If ``s`` is an instance of ``text_type``, return
    ``s.encode(encoding, errors)``, otherwise return ``s``
    '''
    return s.encode(encoding, errors) if isinstance(s, text_type) else s


def ascii_native_(s):
    '''
    Python 3: If ``s`` is an instance of ``text_type``, return
    ``s.encode('ascii')``, otherwise return ``str(s, 'ascii', 'strict')``

    Python 2: If ``s`` is an instance of ``text_type``, return
    ``s.encode('ascii')``, otherwise return ``str(s)``
    '''
    if isinstance(s, text_type):
        s = s.encode('ascii')

    return str(s, 'ascii', 'strict') if PY3 else s


def native_(s, encoding='latin-1', errors='strict'):
    '''
    Python 3: If ``s`` is an instance of ``text_type``, return ``s``, otherwise
    return ``str(s, encoding, errors)``

    Python 2: If ``s`` is an instance of ``text_type``, return
    ``s.encode(encoding, errors)``, otherwise return ``str(s)``
    '''
    if PY3:
        out = s if isinstance(s, text_type) else str(s, encoding, errors)
    else:
        out = s.encode(encoding, errors) if isinstance(s, text_type) else str(s)

    return out


def string_io(data=None):  # cStringIO can't handle unicode
    '''
    Pass data through to stringIO module and return result
    '''
    try:
        return cStringIO(bytes(data))
    except (UnicodeEncodeError, TypeError):
        return StringIO(data)


try:
    if PY3:
        import ipaddress
    else:
        import salt.ext.ipaddress as ipaddress
except ImportError:
    ipaddress = None


class IPv6AddressScoped(ipaddress.IPv6Address):
    '''
    Represent and manipulate single IPv6 Addresses.
    Scope-aware version
    '''
    def __init__(self, address):
        '''
        Instantiate a new IPv6 address object. Scope is moved to an attribute 'scope'.

        Args:
            address: A string or integer representing the IP

              Additionally, an integer can be passed, so
              IPv6Address('2001:db8::') == IPv6Address(42540766411282592856903984951653826560)
              or, more generally
              IPv6Address(int(IPv6Address('2001:db8::'))) == IPv6Address('2001:db8::')

        Raises:
            AddressValueError: If address isn't a valid IPv6 address.

        :param address:
        '''
        # pylint: disable-all
        if not hasattr(self, '_is_packed_binary'):
            # This method (below) won't be around for some Python 3 versions
            # and we need check this differently anyway
            self._is_packed_binary = lambda p: isinstance(p, bytes)
        # pylint: enable-all

        if isinstance(address, string_types) and '%' in address:
            buff = address.split('%')
            if len(buff) != 2:
                raise SaltException('Invalid IPv6 address: "{}"'.format(address))
            address, self.__scope = buff
        else:
            self.__scope = None

        if sys.version_info.major == 2:
            ipaddress._BaseAddress.__init__(self, address)
            ipaddress._BaseV6.__init__(self, address)
        else:
            # Python 3.4 fix. Versions higher are simply not affected
            # https://github.com/python/cpython/blob/3.4/Lib/ipaddress.py#L543-L544
            self._version = 6
            self._max_prefixlen = ipaddress.IPV6LENGTH

        # Efficient constructor from integer.
        if isinstance(address, integer_types):
            self._check_int_address(address)
            self._ip = address
        elif self._is_packed_binary(address):
            self._check_packed_address(address, 16)
            self._ip = ipaddress._int_from_bytes(address, 'big')
        else:
            address = str(address)
            if '/' in address:
                raise ipaddress.AddressValueError("Unexpected '/' in {}".format(address))
            self._ip = self._ip_int_from_string(address)

    def _is_packed_binary(self, data):
        '''
        Check if data is hexadecimal packed

        :param data:
        :return:
        '''
        packed = False
        if len(data) == 16 and ':' not in data:
            try:
                packed = bool(int(str(bytearray(data)).encode('hex'), 16))
            except ValueError:
                pass

        return packed

    @property
    def scope(self):
        '''
        Return scope of IPv6 address.

        :return:
        '''
        return self.__scope

    def __str__(self):
        return text_type(self._string_from_ip_int(self._ip) +
                         ('%' + self.scope if self.scope is not None else ''))


class IPv6InterfaceScoped(ipaddress.IPv6Interface, IPv6AddressScoped):
    '''
    Update
    '''
    def __init__(self, address):
        if isinstance(address, (bytes, int)):
            IPv6AddressScoped.__init__(self, address)
            self.network = ipaddress.IPv6Network(self._ip)
            self._prefixlen = self._max_prefixlen
            return

        addr = ipaddress._split_optional_netmask(address)
        IPv6AddressScoped.__init__(self, addr[0])
        self.network = ipaddress.IPv6Network(address, strict=False)
        self.netmask = self.network.netmask
        self._prefixlen = self.network._prefixlen
        self.hostmask = self.network.hostmask


if ipaddress:
    ipaddress.IPv6Address = IPv6AddressScoped
    if sys.version_info.major == 2:
        ipaddress.IPv6Interface = IPv6InterfaceScoped



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os

from tests.support.unit import skipIf

import salt.utils.platform
import salt.utils.files


class JinjaFiltersTest(object):
    '''
    testing Jinja filters are available via state system
    '''

    def test_data_compare_dicts(self):
        '''
        test jinja filter data.compare_dicts
        '''
        _expected = {'ret': {'a': {'new': 'c', 'old': 'b'}}}

        ret = self.run_function('state.sls',
                                ['jinja_filters.data_compare_dicts'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_data_compare_lists(self):
        '''
        test jinja filter data.compare_list
        '''
        _expected = {'ret': {'old': 'b'}}
        ret = self.run_function('state.sls',
                                ['jinja_filters.data_compare_lists'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_data_decode_dict(self):
        '''
        test jinja filter data.decode_dict
        '''
        _expected = {'ret': {'a': 'b', 'c': 'd'}}
        ret = self.run_function('state.sls',
                                ['jinja_filters.data_decode_dict'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertIn('ret', ret['module_|-test_|-test.echo_|-run']['changes'])

    def test_data_decode_list(self):
        '''
        test jinja filter data.decode_list
        '''
        _expected = {'ret': ['a', 'b', 'c', 'd']}
        ret = self.run_function('state.sls',
                                ['jinja_filters.data_decode_list'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertIn('ret', ret['module_|-test_|-test.echo_|-run']['changes'])

    def test_data_encode_dict(self):
        '''
        test jinja filter data.encode_dict
        '''
        _expected = {'ret': {'a': 'b', 'c': 'd'}}
        ret = self.run_function('state.sls',
                                ['jinja_filters.data_encode_dict'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertIn('ret', ret['module_|-test_|-test.echo_|-run']['changes'])

    def test_data_encode_list(self):
        '''
        test jinja filter data.encode_list
        '''
        _expected = {'ret': ['a', 'b', 'c', 'd']}
        ret = self.run_function('state.sls',
                                ['jinja_filters.data_encode_list'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertIn('ret', ret['module_|-test_|-test.echo_|-run']['changes'])

    def test_data_exactly_n(self):
        '''
        test jinja filter data.exactly_n
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.data_exactly_n'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_data_exactly_one(self):
        '''
        test jinja filter data.exactly_one
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.data_exactly_one'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_data_is_iter(self):
        '''
        test jinja filter data.is_iter
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.data_is_iter'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_data_is_list(self):
        '''
        test jinja filter data.is_list
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.data_is_list'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_data_mysql_to_dict(self):
        '''
        test jinja filter data.mysql_to_dict
        '''
        _expected = {'ret': {'show processlist': {'Info': 'show processlist', 'db': 'NULL', 'Host': 'localhost', 'State': 'init', 'Command': 'Query', 'User': 'root', 'Time': 0, 'Id': 7}}}
        ret = self.run_function('state.sls',
                                ['jinja_filters.data_mysql_to_dict'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_data_sorted_ignorecase(self):
        '''
        test jinja filter data.softed_ignorecase
        '''
        _expected = {'ret': ['Abcd', 'efgh', 'Ijk', 'lmno', 'Pqrs']}
        ret = self.run_function('state.sls',
                                ['jinja_filters.data_sorted_ignorecase'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_data_substring_in_list(self):
        '''
        test jinja filter data.substring_in_list
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.data_substring_in_list'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_dateutils_strftime(self):
        '''
        test jinja filter datautils.strftime
        '''
        ret = self.run_function('state.sls',
                                ['jinja_filters.dateutils_strftime'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertIn('ret', ret['module_|-test_|-test.echo_|-run']['changes'])

    def test_files_is_binary(self):
        '''
        test jinja filter files.is_binary
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.files_is_binary'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_files_is_empty(self):
        '''
        test jinja filter files.is_empty
        '''
        try:
            if salt.utils.platform.is_windows():
                with salt.utils.files.fopen('c:\\empty_file', 'w') as fp:
                    pass
            _expected = {'ret': True}
            ret = self.run_function('state.sls',
                                    ['jinja_filters.files_is_empty'])
            self.assertIn('module_|-test_|-test.echo_|-run', ret)
            self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                             _expected)
        finally:
            if salt.utils.platform.is_windows():
                os.remove('c:\\empty_file')

    def test_files_is_text(self):
        '''
        test jinja filter files.is_text
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.files_is_text'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_files_list_files(self):
        '''
        test jinja filter files.list_files
        '''
        ret = self.run_function('state.sls',
                                ['jinja_filters.files_list_files'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        if salt.utils.platform.is_windows():
            self.assertIn('c:\\\\salt\\\\conf\\\\minion',
                          ret['module_|-test_|-test.echo_|-run']['changes']['ret'])
        else:
            self.assertIn('/bin/ls',
                          ret['module_|-test_|-test.echo_|-run']['changes']['ret'])

    def test_hashutils_base4_64decode(self):
        '''
        test jinja filter hashutils.base64_64decode
        '''
        _expected = {'ret': 'Salt Rocks!'}
        ret = self.run_function('state.sls',
                                ['jinja_filters.hashutils_base4_64decode'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_hashutils_base4_64encode(self):
        '''
        test jinja filter hashutils.base64_64encode
        '''
        _expected = {'ret': 'U2FsdCBSb2NrcyE='}
        ret = self.run_function('state.sls',
                                ['jinja_filters.hashutils_base4_64encode'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_hashutils_file_hashsum(self):
        '''
        test jinja filter hashutils.file_hashsum
        '''
        ret = self.run_function('state.sls',
                                ['jinja_filters.hashutils_file_hashsum'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertIn('ret', ret['module_|-test_|-test.echo_|-run']['changes'])

    def test_hashutils_hmac(self):
        '''
        test jinja filter hashutils.hmac
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.hashutils_hmac'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_hashutils_md5_digest(self):
        '''
        test jinja filter hashutils.md5_digest
        '''
        _expected = {'ret': '85d6e71db655ee8e42c8b18475f0925f'}
        ret = self.run_function('state.sls',
                                ['jinja_filters.hashutils_md5_digest'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_hashutils_random_hash(self):
        '''
        test jinja filter hashutils.random_hash
        '''
        ret = self.run_function('state.sls',
                                ['jinja_filters.hashutils_random_hash'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertIn('ret',
                      ret['module_|-test_|-test.echo_|-run']['changes'])

    def test_hashutils_sha256_digest(self):
        '''
        test jinja filter hashutils.sha256_digest
        '''
        _expected = {'ret': 'cce7fe00fd9cc6122fd3b2ed22feae215bcfe7ac4a7879d336c1993426a763fe'}
        ret = self.run_function('state.sls',
                                ['jinja_filters.hashutils_sha256_digest'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_hashutils_sha512_digest(self):
        '''
        test jinja filter hashutils.sha512_digest
        '''
        _expected = {'ret': '44d829491d8caa7039ad08a5b7fa9dd0f930138c614411c5326dd4755fdc9877ec6148219fccbe404139e7bb850e77237429d64f560c204f3697ab489fd8bfa5'}
        ret = self.run_function('state.sls',
                                ['jinja_filters.hashutils_sha512_digest'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_http_query(self):
        '''
        test jinja filter http.query
        '''
        _expected = {'ret': {}}
        ret = self.run_function('state.sls',
                                ['jinja_filters.http_query'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_avg(self):
        '''
        test jinja filter jinja.avg
        '''
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_avg'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertIn('ret', ret['module_|-test_|-test.echo_|-run']['changes'])

    def test_jinja_difference(self):
        '''
        test jinja filter jinja.difference
        '''
        _expected = {'ret': [1, 3]}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_difference'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_intersect(self):
        '''
        test jinja filter jinja.intersect
        '''
        _expected = {'ret': [2, 4]}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_intersect'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_max(self):
        '''
        test jinja filter jinja.max
        '''
        _expected = {'ret': 4}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_max'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_min(self):
        '''
        test jinja filter jinja.min
        '''
        _expected = {'ret': 1}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_min'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_quote(self):
        '''
        test jinja filter jinja.quote
        '''
        _expected = {'ret': 'Salt Rocks!'}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_quote'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_regex_escape(self):
        '''
        test jinja filter jinja.regex_escape
        '''
        _expected = {'ret': 'Salt\\ Rocks'}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_regex_escape'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_regex_match(self):
        '''
        test jinja filter jinja.regex_match
        '''
        _expected = {'ret': "('a', 'd')"}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_regex_match'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_regex_replace(self):
        '''
        test jinja filter jinja.regex_replace
        '''
        _expected = {'ret': 'lets__replace__spaces'}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_regex_replace'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_regex_search(self):
        '''
        test jinja filter jinja.regex_search
        '''
        _expected = {'ret': "('a', 'd')"}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_regex_search'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_sequence(self):
        '''
        test jinja filter jinja.sequence
        '''
        _expected = {'ret': ['Salt Rocks!']}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_sequence'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_skip(self):
        '''
        test jinja filter jinja.skip
        '''
        _expected = {'ret': None}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_skip'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_symmetric_difference(self):
        '''
        test jinja filter jinja.symmetric_difference
        '''
        _expected = {'ret': [1, 3, 6]}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_symmetric_difference'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_to_bool(self):
        '''
        test jinja filter jinja.to_bool
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_to_bool'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_union(self):
        '''
        test jinja filter jinja.union
        '''
        _expected = {'ret': [1, 2, 3, 4, 6]}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_union'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_unique(self):
        '''
        test jinja filter jinja.unique
        '''
        _expected = {'ret': ['a', 'b', 'c']}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_unique'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_jinja_uuid(self):
        '''
        test jinja filter jinja.uuid
        '''
        _expected = {'ret': '799192d9-7f32-5227-a45f-dfeb4a34e06f'}
        ret = self.run_function('state.sls',
                                ['jinja_filters.jinja_uuid'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_network_gen_mac(self):
        '''
        test jinja filter network.gen_mac
        '''
        _expected = 'AC:DE:48:'
        ret = self.run_function('state.sls',
                                ['jinja_filters.network_gen_mac'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertTrue(ret['module_|-test_|-test.echo_|-run']['changes']['ret'].startswith(_expected))

    def test_network_ipaddr(self):
        '''
        test jinja filter network.ipaddr
        '''
        ret = self.run_function('state.sls',
                                ['jinja_filters.network_ipaddr'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertIn('ret', ret['module_|-test_|-test.echo_|-run']['changes'])

    def test_network_ip_host(self):
        '''
        test jinja filter network.ip_host
        '''
        _expected = {'ret': '192.168.0.12/24'}
        ret = self.run_function('state.sls',
                                ['jinja_filters.network_ip_host'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_network_ipv4(self):
        '''
        test jinja filter network.ipv4
        '''
        _expected = {'ret': ['127.0.0.1']}
        ret = self.run_function('state.sls',
                                ['jinja_filters.network_ipv4'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_network_ipv6(self):
        '''
        test jinja filter network.ipv6
        '''
        ret = self.run_function('state.sls',
                                ['jinja_filters.network_ipv6'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertIn('ret', ret['module_|-test_|-test.echo_|-run']['changes'])

    def test_network_is_ip(self):
        '''
        test jinja filter network.is_ip
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.network_is_ip'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_network_is_ipv4(self):
        '''
        test jinja filter network.is_ipv4
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.network_is_ipv4'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_network_is_ipv6(self):
        '''
        test jinja filter network.is_ipv6
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.network_is_ipv6'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_network_network_hosts(self):
        '''
        test jinja filter network.network_hosts
        '''
        _expected = {'ret': ['192.168.1.1', '192.168.1.2']}
        ret = self.run_function('state.sls',
                                ['jinja_filters.network_network_hosts'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_network_network_size(self):
        '''
        test jinja filter network.network_size
        '''
        _expected = {'ret': 16}
        ret = self.run_function('state.sls',
                                ['jinja_filters.network_network_size'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_path_join(self):
        '''
        test jinja filter path.join
        '''
        _expected = {'ret': os.path.sep + os.path.join('a', 'b', 'c', 'd')}
        ret = self.run_function('state.sls',
                                ['jinja_filters.path_join'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_path_which(self):
        '''
        test jinja filter path.which
        '''
        ret = self.run_function('state.sls',
                                ['jinja_filters.path_which'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertIn('ret', ret['module_|-test_|-test.echo_|-run']['changes'])

    def test_stringutils_contains_whitespace(self):
        '''
        test jinja filter stringutils.contains_whitespace
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.stringutils_contains_whitespace'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_stringutils_is_hex(self):
        '''
        test jinja filter stringutils.is_hex
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.stringutils_is_hex'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_stringutils_random_str(self):
        '''
        test jinja filter stringutils.random_str
        '''
        ret = self.run_function('state.sls',
                                ['jinja_filters.stringutils_random_str'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertIn('ret', ret['module_|-test_|-test.echo_|-run']['changes'])

    def test_stringutils_to_bytes(self):
        '''
        test jinja filter stringutils.to_bytes
        '''
        _expected = {'ret': 'Salt Rocks!'}
        ret = self.run_function('state.sls',
                                ['jinja_filters.stringutils_to_bytes'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertIn('ret', ret['module_|-test_|-test.echo_|-run']['changes'])

    def test_stringutils_to_num(self):
        '''
        test jinja filter stringutils.to_num
        '''
        _expected = {'ret': 42}
        ret = self.run_function('state.sls',
                                ['jinja_filters.stringutils_to_num'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_stringutils_whitelist_blacklist(self):
        '''
        test jinja filter stringutils.whitelist_blacklist
        '''
        _expected = {'ret': True}
        ret = self.run_function('state.sls',
                                ['jinja_filters.stringutils_whitelist_blacklist'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    @skipIf(salt.utils.platform.is_windows(), 'Skip on windows')
    def test_user_get_uid(self):
        '''
        test jinja filter user.get_uid
        '''
        _expected = {'ret': 0}
        ret = self.run_function('state.sls',
                                ['jinja_filters.user_get_uid'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_yamlencoding_yaml_dquote(self):
        '''
        test jinja filter yamlencoding.yaml_dquote
        '''
        _expected = {'ret': 'A double-quoted string in YAML'}
        ret = self.run_function('state.sls',
                                ['jinja_filters.yamlencoding_yaml_dquote'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_yamlencoding_yaml_encode(self):
        '''
        test jinja filter yamlencoding.yaml_encode
        '''
        _expected = {'ret': 'An encoded string in YAML'}
        ret = self.run_function('state.sls',
                                ['jinja_filters.yamlencoding_yaml_encode'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_yamlencoding_yaml_squote(self):
        '''
        test jinja filter yamlencoding.yaml_squote
        '''
        _expected = {'ret': 'A single-quoted string in YAML'}
        ret = self.run_function('state.sls',
                                ['jinja_filters.yamlencoding_yaml_squote'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_yaml(self):
        '''
        test yaml filter
        '''
        _expected = {'ret': "{Question: 'Quieres Café?'}"}
        ret = self.run_function('state.sls', ['jinja_filters.yaml'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)

    def test_json(self):
        '''
        test json filter
        '''
        _expected = {'ret': '{"Question": "Quieres Café?"}'}
        ret = self.run_function('state.sls', ['jinja_filters.json'])
        self.assertIn('module_|-test_|-test.echo_|-run', ret)
        self.assertEqual(ret['module_|-test_|-test.echo_|-run']['changes'],
                         _expected)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import copy
import os
import shutil
import tempfile
import textwrap
import time

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.paths import TMP, TMP_CONF_DIR
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    Mock,
    MagicMock,
    patch,
    mock_open,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.config
import salt.loader
import salt.state
import salt.utils.args
import salt.utils.files
import salt.utils.json
import salt.utils.hashutils
import salt.utils.odict
import salt.utils.platform
import salt.utils.state
import salt.modules.state as state
from salt.exceptions import CommandExecutionError, SaltInvocationError
import salt.modules.config as config
from salt.ext import six


class MockState(object):
    '''
        Mock class
    '''
    def __init__(self):
        pass

    class State(object):
        '''
            Mock state class
        '''
        flag = None

        def __init__(self,
                     opts,
                     pillar_override=False,
                     pillar_enc=None,
                     initial_pillar=None):
            pass

        def verify_data(self, data):
            '''
                Mock verify_data method
            '''
            data = data
            if self.flag:
                return True
            else:
                return False

        @staticmethod
        def call(data):
            '''
                Mock call method
            '''
            data = data
            return list

        @staticmethod
        def call_high(data, orchestration_jid=None):
            '''
                Mock call_high method
            '''
            data = data
            return True

        @staticmethod
        def call_template_str(data):
            '''
                Mock call_template_str method
            '''
            data = data
            return True

        @staticmethod
        def _mod_init(data):
            '''
                Mock _mod_init method
            '''
            data = data
            return True

        def verify_high(self, data):
            '''
                Mock verify_high method
            '''
            data = data
            if self.flag:
                return True
            else:
                return -1

        @staticmethod
        def compile_high_data(data):
            '''
                Mock compile_high_data
            '''
            data = data
            return [{"__id__": "ABC"}]

        @staticmethod
        def call_chunk(data, data1, data2):
            '''
                Mock call_chunk method
            '''
            data = data
            data1 = data1
            data2 = data2
            return {'': 'ABC'}

        @staticmethod
        def call_chunks(data):
            '''
                Mock call_chunks method
            '''
            data = data
            return True

        @staticmethod
        def call_listen(data, ret):
            '''
                Mock call_listen method
            '''
            data = data
            ret = ret
            return True

        def requisite_in(self, data):  # pylint: disable=unused-argument
            return data, []

    class HighState(object):
        '''
            Mock HighState class
        '''
        flag = False
        opts = {'state_top': '',
                'pillar': {}}

        def __init__(self, opts, pillar_override=None, *args, **kwargs):
            self.building_highstate = salt.utils.odict.OrderedDict
            self.state = MockState.State(opts,
                                         pillar_override=pillar_override)

        def render_state(self, sls, saltenv, mods, matches, local=False):
            '''
                Mock render_state method
            '''
            sls = sls
            saltenv = saltenv
            mods = mods
            matches = matches
            local = local
            if self.flag:
                return {}, True
            else:
                return {}, False

        @staticmethod
        def get_top():
            '''
                Mock get_top method
            '''
            return "_top"

        def verify_tops(self, data):
            '''
                Mock verify_tops method
            '''
            data = data
            if self.flag:
                return ["a", "b"]
            else:
                return []

        @staticmethod
        def top_matches(data):
            '''
                Mock top_matches method
            '''
            data = data
            return ["a", "b", "c"]

        @staticmethod
        def push_active():
            '''
                Mock push_active method
            '''
            return True

        @staticmethod
        def compile_highstate():
            '''
                Mock compile_highstate method
            '''
            return "A"

        @staticmethod
        def compile_state_usage():
            '''
                Mock compile_state_usage method
            '''
            return "A"

        @staticmethod
        def pop_active():
            '''
                Mock pop_active method
            '''
            return True

        @staticmethod
        def compile_low_chunks():
            '''
                Mock compile_low_chunks method
            '''
            return True

        def render_highstate(self, data):
            '''
                Mock render_highstate method
            '''
            data = data
            if self.flag:
                return ["a", "b"], True
            else:
                return ["a", "b"], False

        @staticmethod
        def call_highstate(exclude, cache, cache_name, force=None,
                           whitelist=None, orchestration_jid=None):
            '''
                Mock call_highstate method
            '''
            exclude = exclude
            cache = cache
            cache_name = cache_name
            force = force
            whitelist = whitelist
            return True


class MockSerial(object):
    '''
        Mock Class
    '''
    def __init__(self):
        pass

    class Serial(object):
        '''
            Mock Serial class
        '''
        def __init__(self, data):
            data = data

        @staticmethod
        def load(data):
            '''
                Mock load method
            '''
            data = data
            return {"A": "B"}

        @staticmethod
        def dump(data, data1):
            '''
                Mock dump method
            '''
            data = data
            data1 = data1
            return True


class MockTarFile(object):
    '''
        Mock tarfile class
    '''
    path = os.sep + "tmp"

    def __init__(self):
        pass

    @staticmethod
    def open(data, data1):
        '''
            Mock open method
        '''
        data = data
        data1 = data1
        return MockTarFile

    @staticmethod
    def getmembers():
        '''
            Mock getmembers method
        '''
        return [MockTarFile]

    @staticmethod
    def extractall(data):
        '''
            Mock extractall method
        '''
        data = data
        return True

    @staticmethod
    def close():
        '''
            Mock close method
        '''
        return True


@skipIf(NO_MOCK, NO_MOCK_REASON)
class StateTestCase(TestCase, LoaderModuleMockMixin):
    '''
        Test case for salt.modules.state
    '''

    def setup_loader_modules(self):
        utils = salt.loader.utils(
            salt.config.DEFAULT_MINION_OPTS,
            whitelist=['state', 'args', 'systemd', 'path', 'platform']
        )
        utils.keys()
        patcher = patch('salt.modules.state.salt.state', MockState())
        patcher.start()
        self.addCleanup(patcher.stop)
        return {
            state: {
                '__opts__': {
                    'cachedir': '/D',
                    'saltenv': None,
                    '__cli': 'salt',
                },
                '__utils__': utils,
                '__salt__': {
                    'config.get': config.get,
                    'config.option': MagicMock(return_value=''),
                }
            },
            config: {
                '__opts__': {},
                '__pillar__': {},
            },

        }

    def test_running(self):
        '''
            Test of checking i fthe state function is already running
        '''
        self.assertEqual(state.running(True), [])

        mock = MagicMock(side_effect=[[{"fun": "state.running", "pid": "4126",
                                        "jid": "20150325123407204096"}], []])
        with patch.dict(state.__salt__,
                        {'saltutil.is_running': mock}
                        ):
            self.assertListEqual(state.running(),
                                 ['The function "state.running"'
                                  ' is running as PID 4126 and '
                                  'was started at 2015, Mar 25 12:34:07.'
                                  '204096 with jid 20150325123407204096'])

            self.assertListEqual(state.running(), [])

    def test_low(self):
        '''
            Test of executing a single low data call
        '''
        mock = MagicMock(side_effect=[False, None, None])
        with patch.object(state, '_check_queue', mock):
            self.assertFalse(state.low({"state": "pkg", "fun": "installed",
                                        "name": "vi"}))

            MockState.State.flag = False
            self.assertEqual(state.low({"state": "pkg", "fun": "installed",
                                        "name": "vi"}), list)

            MockState.State.flag = True
            self.assertTrue(state.low({"state": "pkg", "fun": "installed",
                                       "name": "vi"}))

    def test_high(self):
        '''
            Test for checking the state system
        '''
        mock = MagicMock(side_effect=[False, None])
        with patch.object(state, '_check_queue', mock):
            self.assertFalse(state.high({"vim": {"pkg": ["installed"]}}))

            mock = MagicMock(return_value={"test": True})
            with patch.object(salt.utils.state, 'get_sls_opts', mock):
                self.assertTrue(state.high({"vim": {"pkg": ["installed"]}}))

    def test_template(self):
        '''
            Test of executing the information
            stored in a template file on the minion
        '''
        mock = MagicMock(side_effect=[False, None, None])
        with patch.object(state, '_check_queue', mock):
            self.assertFalse(state.template('/home/salt/salt.sls'))

            MockState.HighState.flag = True
            self.assertTrue(state.template('/home/salt/salt.sls'))

            MockState.HighState.flag = False
            self.assertTrue(state.template('/home/salt/salt.sls'))

    def test_template_str(self):
        '''
            Test for Executing the information
            stored in a string from an sls template
        '''
        mock = MagicMock(side_effect=[False, None])
        with patch.object(state, '_check_queue', mock):
            self.assertFalse(state.template_str('Template String'))

            self.assertTrue(state.template_str('Template String'))

    def test_apply_(self):
        '''
            Test to apply states
        '''
        mock = MagicMock(return_value=True)
        with patch.object(state, 'sls', mock):
            self.assertTrue(state.apply_(True))

        with patch.object(state, 'highstate', mock):
            self.assertTrue(state.apply_(None))

    def test_list_disabled(self):
        '''
            Test to list disabled states
        '''
        mock = MagicMock(return_value=["A", "B", "C"])
        with patch.dict(state.__salt__, {'grains.get': mock}):
            self.assertListEqual(state.list_disabled(), ["A", "B", "C"])

    def test_enable(self):
        '''
            Test to Enable state function or sls run
        '''
        mock = MagicMock(return_value=["A", "B"])
        with patch.dict(state.__salt__, {'grains.get': mock}):
            mock = MagicMock(return_value=[])
            with patch.dict(state.__salt__, {'grains.setval': mock}):
                mock = MagicMock(return_value=[])
                with patch.dict(state.__salt__, {'saltutil.refresh_modules':
                                                 mock}):
                    self.assertDictEqual(state.enable("A"),
                                         {'msg': 'Info: A state enabled.',
                                          'res': True})

                    self.assertDictEqual(state.enable("Z"),
                                         {'msg': 'Info: Z state already '
                                          'enabled.', 'res': True})

    def test_disable(self):
        '''
            Test to disable state run
        '''
        mock = MagicMock(return_value=["C", "D"])
        with patch.dict(state.__salt__, {'grains.get': mock}):
            mock = MagicMock(return_value=[])
            with patch.dict(state.__salt__, {'grains.setval': mock}):
                mock = MagicMock(return_value=[])
                with patch.dict(state.__salt__, {'saltutil.refresh_modules':
                                                 mock}):
                    self.assertDictEqual(state.disable("C"),
                                         {'msg': 'Info: C state '
                                          'already disabled.',
                                          'res': True})

                    self.assertDictEqual(state.disable("Z"),
                                         {'msg': 'Info: Z state '
                                          'disabled.', 'res': True})

    def test_clear_cache(self):
        '''
            Test to clear out cached state file
        '''
        mock = MagicMock(return_value=["A.cache.p", "B.cache.p", "C"])
        with patch.object(os, 'listdir', mock):
            mock = MagicMock(return_value=True)
            with patch.object(os.path, 'isfile', mock):
                mock = MagicMock(return_value=True)
                with patch.object(os, 'remove', mock):
                    self.assertEqual(state.clear_cache(),
                                     ['A.cache.p',
                                      'B.cache.p'])

    def test_single(self):
        '''
            Test to execute single state function
        '''
        ret = {'pkg_|-name=vim_|-name=vim_|-installed': list}
        mock = MagicMock(side_effect=["A", None, None, None, None])
        with patch.object(state, '_check_queue', mock):
            self.assertEqual(state.single("pkg.installed",
                                          " name=vim"), "A")

            self.assertEqual(state.single("pk", "name=vim"),
                             "Invalid function passed")

            with patch.dict(state.__opts__, {"test": "install"}):
                mock = MagicMock(return_value={"test": ""})
                with patch.object(salt.utils.state, 'get_sls_opts', mock):
                    mock = MagicMock(return_value=True)
                    with patch.object(salt.utils.args, 'test_mode', mock):
                        self.assertRaises(SaltInvocationError,
                                          state.single,
                                          "pkg.installed",
                                          "name=vim",
                                          pillar="A")

                        MockState.State.flag = True
                        self.assertTrue(state.single("pkg.installed",
                                                     "name=vim"))

                        MockState.State.flag = False
                        self.assertDictEqual(state.single("pkg.installed",
                                                          "name=vim"), ret)

    def test_show_top(self):
        '''
            Test to return the top data that the minion will use for a highstate
        '''
        mock = MagicMock(side_effect=["A", None, None])
        with patch.object(state, '_check_queue', mock):
            self.assertEqual(state.show_top(), "A")

            MockState.HighState.flag = True
            self.assertListEqual(state.show_top(), ['a', 'b'])

            MockState.HighState.flag = False
            self.assertListEqual(state.show_top(), ['a', 'b', 'c'])

    def test_run_request(self):
        '''
            Test to Execute the pending state request
        '''
        mock = MagicMock(side_effect=[{},
                                      {"name": "A"},
                                      {"name": {'mods': "A",
                                                'kwargs': {}}}])
        with patch.object(state, 'check_request', mock):
            self.assertDictEqual(state.run_request("A"), {})

            self.assertDictEqual(state.run_request("A"), {})

            mock = MagicMock(return_value=["True"])
            with patch.object(state, 'apply_', mock):
                mock = MagicMock(return_value="")
                with patch.object(os, 'remove', mock):
                    self.assertListEqual(state.run_request("name"),
                                         ["True"])

    def test_show_highstate(self):
        '''
            Test to retrieve the highstate data from the salt master
        '''
        mock = MagicMock(side_effect=["A", None, None])
        with patch.object(state, '_check_queue', mock):
            self.assertEqual(state.show_highstate(), "A")

            self.assertRaises(SaltInvocationError,
                              state.show_highstate,
                              pillar="A")

            self.assertEqual(state.show_highstate(), "A")

    def test_show_lowstate(self):
        '''
            Test to list out the low data that will be applied to this minion
        '''
        mock = MagicMock(side_effect=["A", None])
        with patch.object(state, '_check_queue', mock):
            self.assertRaises(AssertionError, state.show_lowstate)

            self.assertTrue(state.show_lowstate())

    def test_show_state_usage(self):
        '''
            Test to list out the state usage that will be applied to this minion
        '''

        mock = MagicMock(side_effect=["A", None, None])
        with patch.object(state, '_check_queue', mock):
            self.assertEqual(state.show_state_usage(), "A")

            self.assertRaises(SaltInvocationError,
                              state.show_state_usage,
                              pillar="A")

            self.assertEqual(state.show_state_usage(), "A")

    def test_sls_id(self):
        '''
            Test to call a single ID from the
            named module(s) and handle all requisites
        '''
        mock = MagicMock(side_effect=["A", None, None, None])
        with patch.object(state, '_check_queue', mock):
            self.assertEqual(state.sls_id("apache", "http"), "A")

            with patch.dict(state.__opts__, {"test": "A"}):
                mock = MagicMock(
                    return_value={'test': True,
                                  'saltenv': None}
                )
                with patch.object(salt.utils.state, 'get_sls_opts', mock):
                    mock = MagicMock(return_value=True)
                    with patch.object(salt.utils.args, 'test_mode', mock):
                        MockState.State.flag = True
                        MockState.HighState.flag = True
                        self.assertEqual(state.sls_id("apache", "http"), 2)

                        MockState.State.flag = False
                        self.assertDictEqual(state.sls_id("ABC", "http"),
                                             {'': 'ABC'})
                        self.assertRaises(SaltInvocationError,
                                          state.sls_id,
                                          "DEF", "http")

    def test_show_low_sls(self):
        '''
            Test to display the low data from a specific sls
        '''
        mock = MagicMock(side_effect=["A", None, None])
        with patch.object(state, '_check_queue', mock):
            self.assertEqual(state.show_low_sls("foo"), "A")

            with patch.dict(state.__opts__, {"test": "A"}):
                mock = MagicMock(
                    return_value={'test': True,
                                  'saltenv': None}
                )
                with patch.object(salt.utils.state, 'get_sls_opts', mock):
                    MockState.State.flag = True
                    MockState.HighState.flag = True
                    self.assertEqual(state.show_low_sls("foo"), 2)

                    MockState.State.flag = False
                    self.assertListEqual(state.show_low_sls("foo"),
                                         [{'__id__': 'ABC'}])

    def test_show_sls(self):
        '''
            Test to display the state data from a specific sls
        '''
        mock = MagicMock(side_effect=["A", None, None, None])
        with patch.object(state, '_check_queue', mock):
            self.assertEqual(state.show_sls("foo"), "A")

            with patch.dict(state.__opts__, {"test": "A"}):
                mock = MagicMock(
                    return_value={'test': True,
                                  'saltenv': None}
                )
                with patch.object(salt.utils.state, 'get_sls_opts', mock):
                    mock = MagicMock(return_value=True)
                    with patch.object(salt.utils.args, 'test_mode', mock):
                        self.assertRaises(SaltInvocationError,
                                          state.show_sls,
                                          "foo",
                                          pillar="A")

                        MockState.State.flag = True
                        self.assertEqual(state.show_sls("foo"), 2)

                        MockState.State.flag = False
                        self.assertListEqual(state.show_sls("foo"),
                                             ['a', 'b'])

    def test_top(self):
        '''
            Test to execute a specific top file
        '''
        ret = ['Pillar failed to render with the following messages:', 'E']
        mock = MagicMock(side_effect=["A", None, None, None])
        with patch.object(state, '_check_queue', mock):
            self.assertEqual(state.top("reverse_top.sls"), "A")

            mock = MagicMock(side_effect=[['E'], None, None])
            with patch.object(state, '_get_pillar_errors', mock):
                with patch.dict(state.__pillar__, {"_errors": ['E']}):
                    self.assertListEqual(state.top("reverse_top.sls"), ret)

                with patch.dict(state.__opts__, {"test": "A"}):
                    mock = MagicMock(return_value={'test': True})
                    with patch.object(salt.utils.state, 'get_sls_opts', mock):
                        mock = MagicMock(return_value=True)
                        with patch.object(salt.utils.args, 'test_mode', mock):
                            self.assertRaises(SaltInvocationError,
                                              state.top,
                                              "reverse_top.sls",
                                              pillar="A")

                            mock = MagicMock(
                                             return_value
                                             =
                                             'salt://reverse_top.sls')
                            with patch.object(os.path, 'join', mock):
                                mock = MagicMock(return_value=True)
                                with patch.object(state, '_set_retcode',
                                                  mock):
                                    self.assertTrue(
                                                    state.
                                                    top("reverse_top.sls "
                                                        "exclude=exclude.sls"))

    def test_highstate(self):
        '''
            Test to retrieve the state data from the
            salt master for the minion and execute it
        '''
        arg = "whitelist=sls1.sls"
        mock = MagicMock(side_effect=[True, False, False, False])
        with patch.object(state, '_disabled', mock):
            self.assertDictEqual(state.highstate("whitelist=sls1.sls"),
                                 {'comment': 'Disabled',
                                  'name': 'Salt highstate run is disabled. '
                                  'To re-enable, run state.enable highstate',
                                  'result': 'False'})

            mock = MagicMock(side_effect=["A", None, None])
            with patch.object(state, '_check_queue', mock):
                self.assertEqual(state.highstate("whitelist=sls1.sls"), "A")

                with patch.dict(state.__opts__, {"test": "A"}):
                    mock = MagicMock(return_value={'test': True})
                    with patch.object(salt.utils.state, 'get_sls_opts', mock):
                        self.assertRaises(SaltInvocationError,
                                          state.highstate,
                                          "whitelist=sls1.sls",
                                          pillar="A")

                        mock = MagicMock(return_value="A")
                        with patch.object(state, '_filter_running',
                                          mock):
                            mock = MagicMock(return_value=True)
                            with patch.object(state, '_filter_running',
                                              mock):
                                mock = MagicMock(return_value=True)
                                with patch.object(salt.payload, 'Serial',
                                                  mock):
                                    with patch.object(os.path,
                                                      'join', mock):
                                        with patch.object(
                                                          state,
                                                          '_set'
                                                          '_retcode',
                                                          mock):
                                            self.assertTrue(state.
                                                            highstate
                                                            (arg))

    def test_clear_request(self):
        '''
            Test to clear out the state execution request without executing it
        '''
        mock = MagicMock(return_value=True)
        with patch.object(salt.payload, 'Serial', mock):
            mock = MagicMock(side_effect=[False, True, True])
            with patch.object(os.path, 'isfile', mock):
                self.assertTrue(state.clear_request("A"))

                mock = MagicMock(return_value=True)
                with patch.object(os, 'remove', mock):
                    self.assertTrue(state.clear_request())

                mock = MagicMock(return_value={})
                with patch.object(state, 'check_request', mock):
                    self.assertFalse(state.clear_request("A"))

    def test_check_request(self):
        '''
            Test to return the state request information
        '''
        with patch('salt.modules.state.salt.payload', MockSerial):
            mock = MagicMock(side_effect=[True, True, False])
            with patch.object(os.path, 'isfile', mock):
                with patch('salt.utils.files.fopen', mock_open()):
                    self.assertDictEqual(state.check_request(), {'A': 'B'})

                with patch('salt.utils.files.fopen', mock_open()):
                    self.assertEqual(state.check_request("A"), 'B')

                self.assertDictEqual(state.check_request(), {})

    def test_request(self):
        '''
            Test to request the local admin execute a state run
        '''
        mock = MagicMock(return_value=True)
        with patch.object(state, 'apply_', mock):
            mock = MagicMock(return_value=True)
            with patch.object(os.path, 'join', mock):
                mock = MagicMock(return_value=
                                 {"test_run": "",
                                  "mods": "",
                                  "kwargs": ""})
                with patch.object(state, 'check_request', mock):
                    mock = MagicMock(return_value=True)
                    with patch.object(os, 'umask', mock):
                        with patch.object(salt.utils.platform, 'is_windows', mock):
                            with patch.dict(state.__salt__, {'cmd.run': mock}):
                                with patch('salt.utils.files.fopen', mock_open()):
                                    mock = MagicMock(return_value=True)
                                    with patch.object(os, 'umask', mock):
                                        self.assertTrue(state.request("A"))

    def test_sls(self):
        '''
            Test to execute a set list of state files from an environment
        '''
        arg = "core,edit.vim dev"
        ret = ['Pillar failed to render with the following messages:', 'E', '1']
        mock = MagicMock(return_value=True)
        with patch.object(state, 'running', mock):
            with patch.dict(state.__context__, {"retcode": 1}):
                self.assertEqual(state.sls("core,edit.vim dev"), True)

        mock = MagicMock(side_effect=[True, True, True, True, True, True])
        with patch.object(state, '_wait', mock):
            mock = MagicMock(side_effect=[["A"], [], [], [], [], []])
            with patch.object(state, '_disabled', mock):
                with patch.dict(state.__context__, {"retcode": 1}):
                    self.assertEqual(
                                     state.sls("core,edit.vim dev",
                                               None,
                                               None,
                                               True),
                                     ["A"])

                mock = MagicMock(side_effect=[['E', '1'], None, None, None, None])
                with patch.object(state, '_get_pillar_errors', mock):
                    with patch.dict(state.__context__, {"retcode": 5}):
                        with patch.dict(state.__pillar__, {"_errors": ['E', '1']}):
                            self.assertListEqual(state.sls("core,edit.vim dev",
                                                           None,
                                                           None,
                                                           True), ret)

                    with patch.dict(state.__opts__, {"test": None}):
                        mock = MagicMock(return_value={"test": "",
                                                       "saltenv": None})
                        with patch.object(salt.utils.state, 'get_sls_opts', mock):
                            mock = MagicMock(return_value=True)
                            with patch.object(salt.utils.args,
                                              'test_mode',
                                              mock):
                                self.assertRaises(
                                                  SaltInvocationError,
                                                  state.sls,
                                                  "core,edit.vim dev",
                                                  None,
                                                  None,
                                                  True,
                                                  pillar="A")

                                mock = MagicMock(return_value="/D/cache.cache.p")
                                with patch.object(os.path,
                                                  'join',
                                                  mock):
                                    mock = MagicMock(return_value=True)
                                    with patch.object(os.path,
                                                      'isfile',
                                                      mock):
                                        with patch(
                                                   'salt.utils.files.fopen',
                                                   mock_open(b'')):
                                            self.assertTrue(
                                                            state.sls(arg,
                                                                      None,
                                                                      None,
                                                                      True,
                                                                      cache
                                                                      =True
                                                                      )
                                                            )

                                    MockState.HighState.flag = True
                                    self.assertTrue(state.sls("core,edit"
                                                              ".vim dev",
                                                              None,
                                                              None,
                                                              True)
                                                    )

                                    MockState.HighState.flag = False
                                    mock = MagicMock(return_value=True)
                                    with patch.object(state,
                                                      '_filter_'
                                                      'running',
                                                      mock):
                                        self.sub_test_sls()

    def test_get_test_value(self):
        '''
        Test _get_test_value when opts contains different values
        '''
        test_arg = 'test'
        with patch.dict(state.__opts__, {test_arg: True}):
            self.assertTrue(state._get_test_value(test=None),
                            msg='Failure when {0} is True in __opts__'.format(test_arg))

        with patch.dict(config.__pillar__, {test_arg: 'blah'}):
            self.assertFalse(state._get_test_value(test=None),
                            msg='Failure when {0} is blah in __opts__'.format(test_arg))

        with patch.dict(config.__pillar__, {test_arg: 'true'}):
            self.assertFalse(state._get_test_value(test=None),
                            msg='Failure when {0} is true in __opts__'.format(test_arg))

        with patch.dict(config.__opts__, {test_arg: False}):
            self.assertFalse(state._get_test_value(test=None),
                            msg='Failure when {0} is False in __opts__'.format(test_arg))

        with patch.dict(config.__opts__, {}):
            self.assertFalse(state._get_test_value(test=None),
                            msg='Failure when {0} does not exist in __opts__'.format(test_arg))

        with patch.dict(config.__pillar__, {test_arg: None}):
            self.assertEqual(state._get_test_value(test=None), None,
                            msg='Failure when {0} is None in __opts__'.format(test_arg))

        with patch.dict(config.__pillar__, {test_arg: True}):
            self.assertTrue(state._get_test_value(test=None),
                            msg='Failure when {0} is True in __pillar__'.format(test_arg))

        with patch.dict(config.__pillar__, {'master': {test_arg: True}}):
            self.assertTrue(state._get_test_value(test=None),
                            msg='Failure when {0} is True in master __pillar__'.format(test_arg))

        with patch.dict(config.__pillar__, {'master': {test_arg: False}}):
            with patch.dict(config.__pillar__, {test_arg: True}):
                self.assertTrue(state._get_test_value(test=None),
                                msg='Failure when {0} is False in master __pillar__ and True in pillar'.format(test_arg))

        with patch.dict(config.__pillar__, {'master': {test_arg: True}}):
            with patch.dict(config.__pillar__, {test_arg: False}):
                self.assertFalse(state._get_test_value(test=None),
                                 msg='Failure when {0} is True in master __pillar__ and False in pillar'.format(test_arg))

        with patch.dict(state.__opts__, {'test': False}):
            self.assertFalse(state._get_test_value(test=None),
                             msg='Failure when {0} is False in __opts__'.format(test_arg))

        with patch.dict(state.__opts__, {'test': False}):
            with patch.dict(config.__pillar__, {'master': {test_arg: True}}):
                self.assertTrue(state._get_test_value(test=None),
                                msg='Failure when {0} is False in __opts__'.format(test_arg))

        with patch.dict(state.__opts__, {}):
            self.assertTrue(state._get_test_value(test=True),
                            msg='Failure when test is True as arg')

    def sub_test_sls(self):
        '''
            Sub function of test_sls
        '''
        mock = MagicMock(return_value=True)
        with patch.object(os.path, 'join', mock):
            with patch.object(os, 'umask', mock):
                mock = MagicMock(return_value=False)
                with patch.object(salt.utils.platform, 'is_windows', mock):
                    mock = MagicMock(return_value=True)
                    with patch.object(os, 'umask', mock):
                        with patch.object(state, '_set_retcode', mock):
                            with patch.dict(state.__opts__,
                                            {"test": True}):
                                with patch('salt.utils.files.fopen', mock_open()):
                                    self.assertTrue(state.sls("core,edit"
                                                              ".vim dev",
                                                              None,
                                                              None,
                                                              True))

    def test_sls_sync(self):
        '''
        Test test.sls with the sync argument

        We're only mocking the sync functions we expect to sync. If any other
        sync functions are run then they will raise a KeyError, which we want
        as it will tell us that we are syncing things we shouldn't.
        '''
        mock_empty_list = MagicMock(return_value=[])
        with patch.object(state, 'running', mock_empty_list), \
                patch.object(state, '_disabled', mock_empty_list), \
                patch.object(state, '_get_pillar_errors', mock_empty_list):

            sync_mocks = {
                'saltutil.sync_modules': Mock(),
                'saltutil.sync_states': Mock(),
            }
            with patch.dict(state.__salt__, sync_mocks):
                state.sls('foo', sync_mods='modules,states')

            for key in sync_mocks:
                call_count = sync_mocks[key].call_count
                expected = 1
                assert call_count == expected, \
                    '{0} called {1} time(s) (expected: {2})'.format(
                        key, call_count, expected
                    )

            # Test syncing all
            sync_mocks = {'saltutil.sync_all': Mock()}
            with patch.dict(state.__salt__, sync_mocks):
                state.sls('foo', sync_mods='all')

            for key in sync_mocks:
                call_count = sync_mocks[key].call_count
                expected = 1
                assert call_count == expected, \
                    '{0} called {1} time(s) (expected: {2})'.format(
                        key, call_count, expected
                    )

            # sync_mods=True should be interpreted as sync_mods=all
            sync_mocks = {'saltutil.sync_all': Mock()}
            with patch.dict(state.__salt__, sync_mocks):
                state.sls('foo', sync_mods=True)

            for key in sync_mocks:
                call_count = sync_mocks[key].call_count
                expected = 1
                assert call_count == expected, \
                    '{0} called {1} time(s) (expected: {2})'.format(
                        key, call_count, expected
                    )

            # Test syncing all when "all" is passed along with module types.
            # This tests that we *only* run a sync_all and avoid unnecessary
            # extra syncing.
            sync_mocks = {'saltutil.sync_all': Mock()}
            with patch.dict(state.__salt__, sync_mocks):
                state.sls('foo', sync_mods='modules,all')

            for key in sync_mocks:
                call_count = sync_mocks[key].call_count
                expected = 1
                assert call_count == expected, \
                    '{0} called {1} time(s) (expected: {2})'.format(
                        key, call_count, expected
                    )

    def test_pkg(self):
        '''
            Test to execute a packaged state run
        '''
        tar_file = os.sep + os.path.join('tmp', 'state_pkg.tgz')
        mock = MagicMock(side_effect=[False, True, True, True, True, True,
            True, True, True, True, True])
        mock_json_loads_true = MagicMock(return_value=[True])
        mock_json_loads_dictlist = MagicMock(return_value=[{"test": ""}])
        with patch.object(os.path, 'isfile', mock), \
                patch('salt.modules.state.tarfile', MockTarFile), \
                patch.object(salt.utils, 'json', mock_json_loads_dictlist):
            self.assertEqual(state.pkg(tar_file, "", "md5"), {})

            mock = MagicMock(side_effect=[False, 0, 0, 0, 0])
            with patch.object(salt.utils.hashutils, 'get_hash', mock):
                # Verify hash
                self.assertDictEqual(state.pkg(tar_file, "", "md5"), {})

                # Verify file outside intended root
                self.assertDictEqual(state.pkg(tar_file, 0, "md5"), {})

                MockTarFile.path = ""
                with patch('salt.utils.files.fopen', mock_open()), \
                        patch.object(salt.utils.json, 'loads', mock_json_loads_true):
                    self.assertEqual(state.pkg(tar_file, 0, "md5"), True)

                MockTarFile.path = ""
                if six.PY2:
                    with patch('salt.utils.files.fopen', mock_open()), \
                            patch.dict(state.__utils__, {'state.check_result': MagicMock(return_value=True)}):
                        self.assertTrue(state.pkg(tar_file, 0, "md5"))
                else:
                    with patch('salt.utils.files.fopen', mock_open()):
                        self.assertTrue(state.pkg(tar_file, 0, "md5"))

    def test_lock_saltenv(self):
        '''
        Tests lock_saltenv in each function which accepts saltenv on the CLI
        '''
        lock_msg = 'lock_saltenv is enabled, saltenv cannot be changed'
        empty_list_mock = MagicMock(return_value=[])
        with patch.dict(state.__opts__, {'lock_saltenv': True}), \
                patch.dict(state.__salt__, {'grains.get': empty_list_mock}), \
                patch.object(state, 'running', empty_list_mock):

            # Test high
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.high(
                    [{"vim": {"pkg": ["installed"]}}], saltenv='base')

            # Test template
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.template('foo', saltenv='base')

            # Test template_str
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.template_str('foo', saltenv='base')

            # Test apply_ with SLS
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.apply_('foo', saltenv='base')

            # Test apply_ with Highstate
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.apply_(saltenv='base')

            # Test highstate
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.highstate(saltenv='base')

            # Test sls
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.sls('foo', saltenv='base')

            # Test top
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.top('foo.sls', saltenv='base')

            # Test show_highstate
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.show_highstate(saltenv='base')

            # Test show_lowstate
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.show_lowstate(saltenv='base')

            # Test sls_id
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.sls_id('foo', 'bar', saltenv='base')

            # Test show_low_sls
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.show_low_sls('foo', saltenv='base')

            # Test show_sls
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.show_sls('foo', saltenv='base')

            # Test show_top
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.show_top(saltenv='base')

            # Test single
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.single('foo.bar', name='baz', saltenv='base')

            # Test pkg
            with self.assertRaisesRegex(CommandExecutionError, lock_msg):
                state.pkg(
                    '/tmp/salt_state.tgz',
                    '760a9353810e36f6d81416366fc426dc',
                    'md5',
                    saltenv='base')

    def test_get_pillar_errors_CC(self):
        '''
        Test _get_pillar_errors function.
        CC: External clean, Internal clean
        :return:
        '''
        for int_pillar, ext_pillar in [({'foo': 'bar'}, {'fred': 'baz'}),
                                       ({'foo': 'bar'}, None),
                                       ({}, {'fred': 'baz'})]:
            with patch('salt.modules.state.__pillar__', int_pillar):
                for opts, res in [({'force': True}, None),
                                  ({'force': False}, None),
                                  ({}, None)]:
                    assert res == state._get_pillar_errors(kwargs=opts, pillar=ext_pillar)

    def test_get_pillar_errors_EC(self):
        '''
        Test _get_pillar_errors function.
        EC: External erroneous, Internal clean
        :return:
        '''
        errors = ['failure', 'everywhere']
        for int_pillar, ext_pillar in [({'foo': 'bar'}, {'fred': 'baz', '_errors': errors}),
                                       ({}, {'fred': 'baz', '_errors': errors})]:
            with patch('salt.modules.state.__pillar__', int_pillar):
                for opts, res in [({'force': True}, None),
                                  ({'force': False}, errors),
                                  ({}, errors)]:
                    assert res == state._get_pillar_errors(kwargs=opts, pillar=ext_pillar)

    def test_get_pillar_errors_EE(self):
        '''
        Test _get_pillar_errors function.
        CC: External erroneous, Internal erroneous
        :return:
        '''
        errors = ['failure', 'everywhere']
        for int_pillar, ext_pillar in [({'foo': 'bar', '_errors': errors}, {'fred': 'baz', '_errors': errors})]:
            with patch('salt.modules.state.__pillar__', int_pillar):
                for opts, res in [({'force': True}, None),
                                  ({'force': False}, errors),
                                  ({}, errors)]:
                    assert res == state._get_pillar_errors(kwargs=opts, pillar=ext_pillar)

    def test_get_pillar_errors_CE(self):
        '''
        Test _get_pillar_errors function.
        CC: External clean, Internal erroneous
        :return:
        '''
        errors = ['failure', 'everywhere']
        for int_pillar, ext_pillar in [({'foo': 'bar', '_errors': errors}, {'fred': 'baz'}),
                                       ({'foo': 'bar', '_errors': errors}, None)]:
            with patch('salt.modules.state.__pillar__', int_pillar):
                for opts, res in [({'force': True}, None),
                                  ({'force': False}, errors),
                                  ({}, errors)]:
                    assert res == state._get_pillar_errors(kwargs=opts, pillar=ext_pillar)


class TopFileMergingCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {
            state: {
                '__opts__': salt.config.minion_config(
                    os.path.join(TMP_CONF_DIR, 'minion')
                ),
                '__salt__': {
                    'saltutil.is_running': MagicMock(return_value=[]),
                },
            },
        }

    def setUp(self):
        self.cachedir = tempfile.mkdtemp(dir=TMP)
        self.fileserver_root = tempfile.mkdtemp(dir=TMP)
        self.addCleanup(shutil.rmtree, self.cachedir, ignore_errors=True)
        self.addCleanup(shutil.rmtree, self.fileserver_root, ignore_errors=True)

        self.saltenvs = ['base', 'foo', 'bar', 'baz']
        self.saltenv_roots = {
            x: os.path.join(self.fileserver_root, x)
            for x in ('base', 'foo', 'bar', 'baz')
        }
        self.base_top_file = os.path.join(self.saltenv_roots['base'], 'top.sls')
        self.dunder_opts = salt.utils.yaml.safe_load(
            textwrap.dedent('''\
                file_client: local
                default_top: base

                file_roots:
                  base:
                    - {base}
                  foo:
                    - {foo}
                  bar:
                    - {bar}
                  baz:
                    - {baz}
                '''.format(**self.saltenv_roots)
            )
        )
        self.dunder_opts['env_order'] = self.saltenvs

        # Write top files for all but the "baz" environment
        for saltenv in self.saltenv_roots:
            os.makedirs(self.saltenv_roots[saltenv])
            if saltenv == 'baz':
                continue
            top_file = os.path.join(self.saltenv_roots[saltenv], 'top.sls')
            with salt.utils.files.fopen(top_file, 'w') as fp_:
                # Add a section for every environment to each top file, with
                # the SLS target prefixed with the current saltenv.
                for env_name in self.saltenvs:
                    fp_.write(textwrap.dedent('''\
                        {env_name}:
                          '*':
                            - {saltenv}_{env_name}
                        '''.format(env_name=env_name, saltenv=saltenv)))

    def tearDown(self):
        time.sleep(1)
        os.remove(self.base_top_file)

    def show_top(self, **kwargs):
        local_opts = copy.deepcopy(self.dunder_opts)
        local_opts.update(kwargs)
        with patch.dict(state.__opts__, local_opts), \
                patch.object(salt.state.State, '_gather_pillar',
                             MagicMock(return_value={})):
            ret = state.show_top()
            # Lazy way of converting ordered dicts to regular dicts. We don't
            # care about dict ordering for these tests.
            return salt.utils.json.loads(salt.utils.json.dumps(ret))

    def use_limited_base_top_file(self):
        '''
        Overwrites the base top file so that it only contains sections for its
        own saltenv.
        '''
        with salt.utils.files.fopen(self.base_top_file, 'w') as fp_:
            fp_.write(textwrap.dedent('''\
                base:
                  '*':
                    - base_base
                '''))
        time.sleep(1)

    def test_merge_strategy_merge(self):
        '''
        Base overrides everything
        '''
        ret = self.show_top(top_file_merging_strategy='merge')
        assert ret == {
            'base': ['base_base'],
            'foo': ['base_foo'],
            'bar': ['base_bar'],
            'baz': ['base_baz'],
        }, ret

    def test_merge_strategy_merge_limited_base(self):
        '''
        Test with a "base" top file containing only a "base" section. The "baz"
        saltenv should not be in the return data because that env doesn't have
        its own top file and there will be no "baz" section in the "base" env's
        top file.

        Next, append a "baz" section to the rewritten top file and we should
        get results for that saltenv in the return data.
        '''
        self.use_limited_base_top_file()
        ret = self.show_top(top_file_merging_strategy='merge')
        assert ret == {
            'base': ['base_base'],
            'foo': ['foo_foo'],
            'bar': ['bar_bar'],
        }, ret

        # Add a "baz" section
        with salt.utils.files.fopen(self.base_top_file, 'a') as fp_:
            fp_.write(textwrap.dedent('''\
                baz:
                  '*':
                    - base_baz
                '''))

        ret = self.show_top(top_file_merging_strategy='merge')
        assert ret == {
            'base': ['base_base'],
            'foo': ['foo_foo'],
            'bar': ['bar_bar'],
            'baz': ['base_baz'],
        }, ret

    def test_merge_strategy_merge_state_top_saltenv_base(self):
        '''
        This tests with state_top_saltenv=base, which should pull states *only*
        from the base saltenv.
        '''
        ret = self.show_top(
            top_file_merging_strategy='merge',
            state_top_saltenv='base')
        assert ret == {
            'base': ['base_base'],
            'foo': ['base_foo'],
            'bar': ['base_bar'],
            'baz': ['base_baz'],
        }, ret

    def test_merge_strategy_merge_state_top_saltenv_foo(self):
        '''
        This tests with state_top_saltenv=foo, which should pull states *only*
        from the foo saltenv. Since that top file is only authoritative for
        its own saltenv, *only* the foo saltenv's matches from the foo top file
        should be in the return data.
        '''
        ret = self.show_top(
            top_file_merging_strategy='merge',
            state_top_saltenv='foo')
        assert ret == {'foo': ['foo_foo']}, ret

    def test_merge_strategy_merge_all(self):
        '''
        Include everything in every top file
        '''
        ret = self.show_top(top_file_merging_strategy='merge_all')
        assert ret == {
            'base': ['base_base', 'foo_base', 'bar_base'],
            'foo': ['base_foo', 'foo_foo', 'bar_foo'],
            'bar': ['base_bar', 'foo_bar', 'bar_bar'],
            'baz': ['base_baz', 'foo_baz', 'bar_baz'],
        }, ret

    def test_merge_strategy_merge_all_alternate_env_order(self):
        '''
        Use an alternate env_order. This should change the order in which the
        SLS targets appear in the result.
        '''
        ret = self.show_top(
            top_file_merging_strategy='merge_all',
            env_order=['bar', 'foo', 'base'])
        assert ret == {
            'base': ['bar_base', 'foo_base', 'base_base'],
            'foo': ['bar_foo', 'foo_foo', 'base_foo'],
            'bar': ['bar_bar', 'foo_bar', 'base_bar'],
            'baz': ['bar_baz', 'foo_baz', 'base_baz'],
        }, ret

    def test_merge_strategy_merge_all_state_top_saltenv_base(self):
        '''
        This tests with state_top_saltenv=base, which should pull states *only*
        from the base saltenv. Since we are using the "merge_all" strategy, all
        the states from that top file should be in the return data.
        '''
        ret = self.show_top(
            top_file_merging_strategy='merge_all',
            state_top_saltenv='base')
        assert ret == {
            'base': ['base_base'],
            'foo': ['base_foo'],
            'bar': ['base_bar'],
            'baz': ['base_baz'],
        }, ret

    def test_merge_strategy_merge_all_state_top_saltenv_foo(self):
        '''
        This tests with state_top_saltenv=foo, which should pull states *only*
        from the foo saltenv. Since we are using the "merge_all" strategy, all
        the states from that top file should be in the return data.
        '''
        ret = self.show_top(
            top_file_merging_strategy='merge_all',
            state_top_saltenv='foo')
        assert ret == {
            'base': ['foo_base'],
            'foo': ['foo_foo'],
            'bar': ['foo_bar'],
            'baz': ['foo_baz'],
        }, ret

    def test_merge_strategy_same(self):
        '''
        Each env should get its SLS targets from its own top file, with the
        "baz" env pulling from "base" since default_top=base and there is no
        top file in the "baz" saltenv.
        '''
        ret = self.show_top(top_file_merging_strategy='same')
        assert ret == {
            'base': ['base_base'],
            'foo': ['foo_foo'],
            'bar': ['bar_bar'],
            'baz': ['base_baz'],
        }, ret

    def test_merge_strategy_same_limited_base(self):
        '''
        Each env should get its SLS targets from its own top file, with the
        "baz" env pulling from "base" since default_top=base and there is no
        top file in the "baz" saltenv.
        '''
        self.use_limited_base_top_file()
        ret = self.show_top(top_file_merging_strategy='same')
        assert ret == {
            'base': ['base_base'],
            'foo': ['foo_foo'],
            'bar': ['bar_bar'],
        }, ret

    def test_merge_strategy_same_default_top_foo(self):
        '''
        Each env should get its SLS targets from its own top file, with the
        "baz" env pulling from "foo" since default_top=foo and there is no top
        file in the "baz" saltenv.
        '''
        ret = self.show_top(
            top_file_merging_strategy='same',
            default_top='foo')
        assert ret == {
            'base': ['base_base'],
            'foo': ['foo_foo'],
            'bar': ['bar_bar'],
            'baz': ['foo_baz'],
        }, ret

    def test_merge_strategy_same_state_top_saltenv_base(self):
        '''
        Test the state_top_saltenv parameter to load states exclusively from
        the base saltenv, with the "same" merging strategy. This should
        result in just the base environment's states from the base top file
        being in the merged result.
        '''
        ret = self.show_top(
            top_file_merging_strategy='same',
            state_top_saltenv='base')
        assert ret == {'base': ['base_base']}, ret

    def test_merge_strategy_same_state_top_saltenv_foo(self):
        '''
        Test the state_top_saltenv parameter to load states exclusively from
        the foo saltenv, with the "same" merging strategy. This should
        result in just the foo environment's states from the foo top file
        being in the merged result.
        '''
        ret = self.show_top(
            top_file_merging_strategy='same',
            state_top_saltenv='foo')
        assert ret == {'foo': ['foo_foo']}, ret

    def test_merge_strategy_same_state_top_saltenv_baz(self):
        '''
        Test the state_top_saltenv parameter to load states exclusively from
        the baz saltenv, with the "same" merging strategy. This should
        result in an empty dictionary since there is no top file in that
        environment.
        '''
        ret = self.show_top(
            top_file_merging_strategy='same',
            state_top_saltenv='baz')
        assert ret == {}, ret



# coding: utf-8

# Python libs
from __future__ import absolute_import

# Salt testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock
from tests.support.mixins import LoaderModuleMockMixin

# Salt libs
import salt.beacons.load as load
import salt.utils.platform

import logging
log = logging.getLogger(__name__)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class LoadBeaconTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test case for salt.beacons.load
    '''

    def setup_loader_modules(self):
        return {
            load: {
                '__context__': {},
                '__salt__': {},
            }
        }

    def test_non_list_config(self):
        config = {}

        ret = load.validate(config)

        self.assertEqual(ret, (False, 'Configuration for load beacon must'
                                      ' be a list.'))

    def test_empty_config(self):
        config = [{}]

        ret = load.validate(config)

        self.assertEqual(ret, (False, 'Averages configuration is required'
                                      ' for load beacon.'))

    @skipIf(salt.utils.platform.is_windows(),
            'os.getloadavg not available on Windows')
    def test_load_match(self):
        with patch('os.getloadavg',
                   MagicMock(return_value=(1.82, 1.84, 1.56))):
            config = [{'averages': {'1m': [0.0, 2.0],
                                    '5m': [0.0, 1.5],
                                    '15m': [0.0, 1.0]},
                       'emitatstartup': True,
                       'onchangeonly': False}]

            ret = load.validate(config)

            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            _expected_return = [{'1m': 1.82, '5m': 1.84, '15m': 1.56}]
            ret = load.beacon(config)
            self.assertEqual(ret, _expected_return)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase
from tests.support.mock import (
    MagicMock,
    patch
)

# Import Salt Libs
from salt.exceptions import CommandExecutionError
import salt.modules.mac_assistive as assistive


class AssistiveTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {assistive: {}}

    def test_install_assistive_bundle(self):
        '''
        Test installing a bundle ID as being allowed to run with assistive access
        '''
        mock_ret = MagicMock(return_value={'retcode': 0})
        with patch.dict(assistive.__salt__, {'cmd.run_all': mock_ret}):
            with patch.dict(assistive.__grains__, {'osrelease': '10.11.3'}):
                self.assertTrue(assistive.install('foo'))

    def test_install_assistive_error(self):
        '''
        Test installing a bundle ID as being allowed to run with assistive access
        '''
        mock_ret = MagicMock(return_value={'retcode': 1})
        with patch.dict(assistive.__salt__, {'cmd.run_all': mock_ret}):
            with patch.dict(assistive.__grains__, {'osrelease': '10.11.3'}):
                self.assertRaises(CommandExecutionError, assistive.install, 'foo')

    def test_installed_bundle(self):
        '''
        Test checking to see if a bundle id is installed as being able to use assistive access
        '''
        with patch('salt.modules.mac_assistive._get_assistive_access',
                   MagicMock(return_value=[('foo', 0)])):
            self.assertTrue(assistive.installed('foo'))

    def test_installed_bundle_not(self):
        '''
        Test checking to see if a bundle id is installed as being able to use assistive access
        '''
        with patch('salt.modules.mac_assistive._get_assistive_access',
                   MagicMock(return_value=[])):
            self.assertFalse(assistive.installed('foo'))

    def test_enable_assistive(self):
        '''
        Test enabling a bundle ID as being allowed to run with assistive access
        '''
        mock_ret = MagicMock(return_value={'retcode': 0})
        with patch.dict(assistive.__salt__, {'cmd.run_all': mock_ret}), \
                patch('salt.modules.mac_assistive._get_assistive_access',
                      MagicMock(return_value=[('foo', 0)])):
            self.assertTrue(assistive.enable('foo', True))

    def test_enable_error(self):
        '''
        Test enabled a bundle ID that throws a command error
        '''
        mock_ret = MagicMock(return_value={'retcode': 1})
        with patch.dict(assistive.__salt__, {'cmd.run_all': mock_ret}), \
                patch('salt.modules.mac_assistive._get_assistive_access',
                      MagicMock(return_value=[('foo', 0)])):
            self.assertRaises(CommandExecutionError,
                              assistive.enable,
                              'foo')

    def test_enable_false(self):
        '''
        Test return of enable function when app isn't found.
        '''
        with patch('salt.modules.mac_assistive._get_assistive_access',
                   MagicMock(return_value=[])):
            self.assertFalse(assistive.enable('foo'))

    def test_enabled_assistive(self):
        '''
        Test enabling a bundle ID as being allowed to run with assistive access
        '''
        with patch('salt.modules.mac_assistive._get_assistive_access',
                   MagicMock(return_value=[('foo', '1')])):
            self.assertTrue(assistive.enabled('foo'))

    def test_enabled_assistive_false(self):
        '''
        Test if a bundle ID is disabled for assistive access
        '''
        with patch('salt.modules.mac_assistive._get_assistive_access',
                   MagicMock(return_value=[])):
            self.assertFalse(assistive.enabled('foo'))

    def test_remove_assistive(self):
        '''
        Test removing an assitive bundle.
        '''
        mock_ret = MagicMock(return_value={'retcode': 0})
        with patch.dict(assistive.__salt__, {'cmd.run_all': mock_ret}):
            self.assertTrue(assistive.remove('foo'))

    def test_remove_assistive_error(self):
        '''
        Test removing an assitive bundle.
        '''
        mock_ret = MagicMock(return_value={'retcode': 1})
        with patch.dict(assistive.__salt__, {'cmd.run_all': mock_ret}):
            self.assertRaises(CommandExecutionError,
                              assistive.remove,
                              'foo')

    def test_get_assistive_access(self):
        '''
        Test if a bundle ID is enabled for assistive access
        '''
        mock_out = 'kTCCServiceAccessibility|/bin/bash|1|1|1|\n' \
                   'kTCCServiceAccessibility|/usr/bin/osascript|1|1|1|'
        mock_ret = MagicMock(return_value={'retcode': 0, 'stdout': mock_out})
        expected = [('/bin/bash', '1'), ('/usr/bin/osascript', '1')]
        with patch.dict(assistive.__salt__, {'cmd.run_all': mock_ret}):
            self.assertEqual(assistive._get_assistive_access(), expected)

    def test_get_assistive_access_error(self):
        '''
        Test a CommandExecutionError is raised when something goes wrong.
        '''
        mock_ret = MagicMock(return_value={'retcode': 1})
        with patch.dict(assistive.__salt__, {'cmd.run_all': mock_ret}):
            self.assertRaises(CommandExecutionError,
                              assistive._get_assistive_access)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.rabbitmq as rabbitmq
from salt.exceptions import CommandExecutionError


@skipIf(NO_MOCK, NO_MOCK_REASON)
class RabbitmqTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.rabbitmq
    '''
    def setup_loader_modules(self):
        return {rabbitmq: {'__context__': {'rabbitmqctl': None, 'rabbitmq-plugins': None}}}

    # 'list_users_rabbitmq2' function tests: 1

    def test_list_users_rabbitmq2(self):
        '''
        Test if it return a list of users based off of rabbitmqctl user_list.
        '''
        mock_run = MagicMock(return_value={
            'retcode': 0,
            'stdout': 'Listing users ...\nguest\t[administrator, user]\njustAnAdmin\t[administrator]\n',
            'stderr': '',
        })
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.list_users(),
                                 {'guest': ['administrator', 'user'], 'justAnAdmin': ['administrator']})

    # 'list_users_rabbitmq3' function tests: 1

    def test_list_users_rabbitmq3(self):
        '''
        Test if it return a list of users based off of rabbitmqctl user_list.
        '''
        mock_run = MagicMock(return_value={
            'retcode': 0,
            'stdout': 'guest\t[administrator user]\r\nother\t[a b]\r\n',
            'stderr': ''
        })
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.list_users(), {'guest': ['administrator', 'user'], 'other': ['a', 'b']})

    # 'list_users_with_warning_rabbitmq2' function tests: 1

    def test_list_users_with_warning_rabbitmq2(self):
        '''
        Test if having a leading WARNING returns the user_list anyway.
        '''
        rtn_stdout = '\n'.join([
            'WARNING: ignoring /etc/rabbitmq/rabbitmq.conf -- location has moved to /etc/rabbitmq/rabbitmq-env.conf',
            'Listing users ...',
            'guest\t[administrator, user]\n',
        ])
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': rtn_stdout, 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.list_users(), {'guest': ['administrator', 'user']})

    # 'list_users_with_warning_rabbitmq3' function tests: 1

    def test_list_users_with_warning_rabbitmq3(self):
        '''
        Test if having a leading WARNING returns the user_list anyway.
        '''
        rtn_stdout = '\n'.join([
            'WARNING: ignoring /etc/rabbitmq/rabbitmq.conf -- location has moved to /etc/rabbitmq/rabbitmq-env.conf',
            'Listing users ...',
            'guest\t[administrator user]\n',
        ])
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': rtn_stdout, 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.list_users(), {'guest': ['administrator', 'user']})

    # 'list_vhosts' function tests: 2

    def test_list_vhosts(self):
        '''
        Test if it return a list of vhost based on rabbitmqctl list_vhosts.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': '/\nsaltstack\n...', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertListEqual(rabbitmq.list_vhosts(), ['/', 'saltstack', '...'])

    def test_list_vhosts_with_warning(self):
        '''
        Test if it return a list of vhost based on rabbitmqctl list_vhosts even with a leading WARNING.
        '''
        rtn_stdout = '\n'.join([
            'WARNING: ignoring /etc/rabbitmq/rabbitmq.conf -- location has moved to /etc/rabbitmq/rabbitmq-env.conf',
            'Listing users ...',
            '/',
            'saltstack',
            '...\n',
        ])
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': rtn_stdout, 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertListEqual(rabbitmq.list_vhosts(), ['/', 'saltstack', '...'])

    # 'user_exists' function tests: 2

    def test_user_exists(self):
        '''
        Test whether a given rabbitmq-internal user exists based
        on rabbitmqctl list_users.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'Listing users ...\n'
                                                                   'saltstack\t[administrator]\n...done', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertTrue(rabbitmq.user_exists('saltstack'))

    def test_user_exists_negative(self):
        '''
        Negative test of whether rabbitmq-internal user exists based
        on rabbitmqctl list_users.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'Listing users ...\n'
                                                                   'saltstack\t[administrator]\n...done', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertFalse(rabbitmq.user_exists('salt'))

    # 'vhost_exists' function tests: 2

    def test_vhost_exists(self):
        '''
        Test if it return whether the vhost exists based
        on rabbitmqctl list_vhosts.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'Listing vhosts ...\nsaltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertTrue(rabbitmq.vhost_exists('saltstack'))

    def test_vhost_exists_negative(self):
        '''
        Test if it return whether the vhost exists based
        on rabbitmqctl list_vhosts.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'Listing vhosts ...\nsaltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertFalse(rabbitmq.vhost_exists('salt'))

    # 'add_user' function tests: 1

    def test_add_user(self):
        '''
        Test if it add a rabbitMQ user via rabbitmqctl
        user_add <user> <password>
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.add_user('saltstack'),
                                 {'Added': 'saltstack'})

        mock_run = MagicMock(return_value='Error')
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            with patch.object(rabbitmq, 'clear_password',
                              return_value={'Error': 'Error', 'retcode': 1}):
                self.assertRaises(CommandExecutionError, rabbitmq.add_user, 'saltstack')

    # 'delete_user' function tests: 1

    def test_delete_user(self):
        '''
        Test if it deletes a user via rabbitmqctl delete_user.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.delete_user('saltstack'),
                                 {'Deleted': 'saltstack'})

    # 'change_password' function tests: 1

    def test_change_password(self):
        '''
        Test if it changes a user's password.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.change_password('saltstack',
                                                          'salt@123'),
                                 {'Password Changed': 'saltstack'})

    # 'clear_password' function tests: 1

    def test_clear_password(self):
        '''
        Test if it removes a user's password.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.clear_password('saltstack'),
                                 {'Password Cleared': 'saltstack'})

    # 'add_vhost' function tests: 1

    def test_add_vhost(self):
        '''
        Test if it adds a vhost via rabbitmqctl add_vhost.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.add_vhost('saltstack'),
                                 {'Added': 'saltstack'})

    # 'delete_vhost' function tests: 1

    def test_delete_vhost(self):
        '''
        Test if it deletes a vhost rabbitmqctl delete_vhost.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.delete_vhost('saltstack'),
                                 {'Deleted': 'saltstack'})

    # 'set_permissions' function tests: 1

    def test_set_permissions(self):
        '''
        Test if it sets permissions for vhost via rabbitmqctl set_permissions.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.set_permissions('myvhost', 'myuser'),
                                 {'Permissions Set': 'saltstack'})

    # 'list_permissions' function tests: 1

    def test_list_permissions(self):
        '''
        Test if it lists permissions for a vhost
        via rabbitmqctl list_permissions.
        '''
        mock_run = MagicMock(return_value={
            'retcode': 0,
            'stdout': 'Listing stuff ...\nsaltstack\tsaltstack\t.*\t1\nguest\t0\tone\n...done',
            'stderr': '',
        })
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.list_user_permissions('myuser'),
                                 {'saltstack': ['saltstack', '.*', '1'], 'guest': ['0', 'one']})

    # 'list_user_permissions' function tests: 1

    def test_list_user_permissions(self):
        '''
        Test if it list permissions for a user
        via rabbitmqctl list_user_permissions.
        '''
        mock_run = MagicMock(return_value={
            'retcode': 0,
            'stdout': 'Listing stuff ...\nsaltstack\tsaltstack\t0\t1\nguest\t0\tone\n...done',
            'stderr': '',
        })
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.list_user_permissions('myuser'),
                                 {'saltstack': ['saltstack', '0', '1'], 'guest': ['0', 'one']})

    # 'set_user_tags' function tests: 1

    def test_set_user_tags(self):
        '''
        Test if it add user tags via rabbitmqctl set_user_tags.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.set_user_tags('myadmin', 'admin'),
                                 {'Tag(s) set': 'saltstack'})

    # 'status' function tests: 1

    def test_status(self):
        '''
        Test if it return rabbitmq status.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertEqual(rabbitmq.status(), 'saltstack')

    # 'cluster_status' function tests: 1

    def test_cluster_status(self):
        '''
        Test if it return rabbitmq cluster_status.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertEqual(rabbitmq.cluster_status(), 'saltstack')

    # 'join_cluster' function tests: 1

    def test_join_cluster(self):
        '''
        Test if it join a rabbit cluster.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.join_cluster('rabbit.example.com'),
                                 {'Join': 'saltstack'})

    # 'stop_app' function tests: 1

    def test_stop_app(self):
        '''
        Test if it stops the RabbitMQ application,
        leaving the Erlang node running.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertEqual(rabbitmq.stop_app(), 'saltstack')

    # 'start_app' function tests: 1

    def test_start_app(self):
        '''
        Test if it start the RabbitMQ application.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertEqual(rabbitmq.start_app(), 'saltstack')

    # 'reset' function tests: 1

    def test_reset(self):
        '''
        Test if it return a RabbitMQ node to its virgin state
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertEqual(rabbitmq.reset(), 'saltstack')

    # 'force_reset' function tests: 1

    def test_force_reset(self):
        '''
        Test if it forcefully Return a RabbitMQ node to its virgin state
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertEqual(rabbitmq.force_reset(), 'saltstack')

    # 'list_queues' function tests: 1

    def test_list_queues(self):
        '''
        Test if it returns queue details of the / virtual host
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack\t0\nceleryev.234-234\t10',
                                           'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.list_queues(), {'saltstack': ['0'], 'celeryev.234-234': ['10']})

    # 'list_queues_vhost' function tests: 1

    def test_list_queues_vhost(self):
        '''
        Test if it returns queue details of specified virtual host.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack\t0\nceleryev.234-234\t10',
                                           'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.list_queues_vhost('consumers'), {'saltstack': ['0'],
                                                                           'celeryev.234-234': ['10']})

    # 'list_policies' function tests: 3

    def test_list_policies(self):
        '''
        Test if it return a dictionary of policies nested by vhost
        and name based on the data returned from rabbitmqctl list_policies.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        mock_pkg = MagicMock(return_value='3.7')
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run, 'pkg.version': mock_pkg}), \
             patch.dict(rabbitmq.__grains__, {'os_family': ''}):
            self.assertDictEqual(rabbitmq.list_policies(), {})

    def test_list_policies_freebsd(self):
        '''
        Test if it return a dictionary of policies nested by vhost
        and name based on the data returned from rabbitmqctl list_policies.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        mock_pkg = MagicMock(return_value='3.7')
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run, 'pkg.version': mock_pkg}), \
             patch.dict(rabbitmq.__grains__, {'os_family': 'FreeBSD'}):
            self.assertDictEqual(rabbitmq.list_policies(), {})

    def test_list_policies_old_version(self):
        '''
        Test if it return a dictionary of policies nested by vhost
        and name based on the data returned from rabbitmqctl list_policies.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        mock_pkg = MagicMock(return_value='3.0')
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run, 'pkg.version': mock_pkg}), \
             patch.dict(rabbitmq.__grains__, {'os_family': ''}):
            self.assertDictEqual(rabbitmq.list_policies(), {})

    # 'set_policy' function tests: 1

    def test_set_policy(self):
        '''
        Test if it set a policy based on rabbitmqctl set_policy.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.set_policy('/', 'HA', '.*',
                                                     '{"ha-mode": "all"}'),
                                 {'Set': 'saltstack'})

    # 'delete_policy' function tests: 1

    def test_delete_policy(self):
        '''
        Test if it delete a policy based on rabbitmqctl clear_policy.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run}):
            self.assertDictEqual(rabbitmq.delete_policy('/', 'HA'),
                                 {'Deleted': 'saltstack'})

    # 'policy_exists' function tests: 1

    def test_policy_exists(self):
        '''
        Test if it return whether the policy exists
        based on rabbitmqctl list_policies.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        mock_pkg = MagicMock(return_value='3.0')
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run, 'pkg.version': mock_pkg}), \
             patch.dict(rabbitmq.__grains__, {'os_family': ''}):
            self.assertFalse(rabbitmq.policy_exists('/', 'HA'))

    # 'list_available_plugins' function tests: 2

    def test_list_available_plugins(self):
        '''
        Test if it returns a list of plugins.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack\nsalt\nother', 'stderr': ''})
        mock_pkg = MagicMock(return_value='')
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run,
                                            'pkg.version': mock_pkg}):
            self.assertListEqual(rabbitmq.list_available_plugins(), ['saltstack', 'salt', 'other'])

    def test_list_available_plugins_space_delimited(self):
        '''
        Test if it returns a list of plugins.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack salt other', 'stderr': ''})
        mock_pkg = MagicMock(return_value='')
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run,
                                            'pkg.version': mock_pkg}):
            self.assertListEqual(rabbitmq.list_available_plugins(), ['saltstack', 'salt', 'other'])

    # 'list_enabled_plugins' function tests: 2

    def test_list_enabled_plugins(self):
        '''
        Test if it returns a list of plugins.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack\nsalt\nother', 'stderr': ''})
        mock_pkg = MagicMock(return_value='')
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run,
                                            'pkg.version': mock_pkg}):
            self.assertListEqual(rabbitmq.list_enabled_plugins(), ['saltstack', 'salt', 'other'])

    def test_list_enabled_plugins_space_delimited(self):
        '''
        Test if it returns a list of plugins.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack salt other', 'stderr': ''})
        mock_pkg = MagicMock(return_value='')
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run,
                                            'pkg.version': mock_pkg}):
            self.assertListEqual(rabbitmq.list_enabled_plugins(), ['saltstack', 'salt', 'other'])

    # 'plugin_is_enabled' function tests: 2

    def test_plugin_is_enabled(self):
        '''
        Test if it returns true for an enabled plugin.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack\nsalt\nother', 'stderr': ''})
        mock_pkg = MagicMock(return_value='')
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run,
                                            'pkg.version': mock_pkg}):
            self.assertTrue(rabbitmq.plugin_is_enabled('saltstack'))
            self.assertTrue(rabbitmq.plugin_is_enabled('salt'))
            self.assertTrue(rabbitmq.plugin_is_enabled('other'))

    def test_plugin_is_enabled_negative(self):
        '''
        Test if it returns false for a disabled plugin.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack\nother', 'stderr': ''})
        mock_pkg = MagicMock(return_value='')
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run,
                                            'pkg.version': mock_pkg}):
            self.assertFalse(rabbitmq.plugin_is_enabled('salt'))
            self.assertFalse(rabbitmq.plugin_is_enabled('stack'))
            self.assertFalse(rabbitmq.plugin_is_enabled('random'))

    # 'enable_plugin' function tests: 1

    def test_enable_plugin(self):
        '''
        Test if it enable a RabbitMQ plugin via the rabbitmq-plugins command.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        mock_pkg = MagicMock(return_value='')
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run,
                                            'pkg.version': mock_pkg}):
            self.assertDictEqual(rabbitmq.enable_plugin('salt'),
                                 {'Enabled': 'saltstack'})

    # 'disable_plugin' function tests: 1

    def test_disable_plugin(self):
        '''
        Test if it disable a RabbitMQ plugin via the rabbitmq-plugins command.
        '''
        mock_run = MagicMock(return_value={'retcode': 0, 'stdout': 'saltstack', 'stderr': ''})
        mock_pkg = MagicMock(return_value='')
        with patch.dict(rabbitmq.__salt__, {'cmd.run_all': mock_run,
                                            'pkg.version': mock_pkg}):
            self.assertDictEqual(rabbitmq.disable_plugin('salt'),
                                 {'Disabled': 'saltstack'})



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON

# Import Salt Libs
import salt.pillar.saltclass as saltclass


base_path = os.path.dirname(os.path.realpath(__file__))
fake_minion_id = 'fake_id'
fake_pillar = {}
fake_args = ({'path': os.path.abspath(
                        os.path.join(base_path, '..', '..', 'integration',
                                     'files', 'saltclass', 'examples'))})
fake_opts = {}
fake_salt = {}
fake_grains = {}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class SaltclassPillarTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.pillar.saltclass
    '''
    def setup_loader_modules(self):
        return {saltclass: {'__opts__': fake_opts,
                            '__salt__': fake_salt,
                            '__grains__': fake_grains
                           }}

    def _runner(self, expected_ret):
        full_ret = {}
        parsed_ret = []
        try:
            full_ret = saltclass.ext_pillar(fake_minion_id, fake_pillar, fake_args)
            parsed_ret = full_ret['__saltclass__']['classes']
        # Fail the test if we hit our NoneType error
        except TypeError as err:
            self.fail(err)
        # Else give the parsed content result
        self.assertListEqual(parsed_ret, expected_ret)

    def test_succeeds(self):
        ret = ['default.users', 'default.motd', 'default.empty', 'default', 'roles.app']
        self._runner(ret)



# -*- coding: utf-8 -*-
'''
Tests for the archive state
'''
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import shutil
import textwrap

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf
from tests.support.paths import TMP
from tests.support.helpers import destructiveTest

# Import salt libs
import salt.utils.files
import salt.utils.path
import salt.utils.platform
import salt.utils.stringutils

# Import 3rd party libs
from salt.ext import six
try:
    import zipfile  # pylint: disable=W0611
    HAS_ZIPFILE = True
except ImportError:
    HAS_ZIPFILE = False


@destructiveTest
class ArchiveTest(ModuleCase):
    '''
    Validate the archive module
    '''
    # Base path used for test artifacts
    base_path = os.path.join(TMP, 'modules', 'archive')

    def _set_artifact_paths(self, arch_fmt):
        '''
        Define the paths for the source, archive, and destination files

        :param str arch_fmt: The archive format used in the test
        '''
        self.src = os.path.join(self.base_path, '{0}_src_dir'.format(arch_fmt))
        self.src_file = os.path.join(self.src, 'file')
        self.arch = os.path.join(self.base_path, 'archive.{0}'.format(arch_fmt))
        self.dst = os.path.join(self.base_path, '{0}_dst_dir'.format(arch_fmt))

    def _set_up(self, arch_fmt, unicode_filename=False):
        '''
        Create source file tree and destination directory

        :param str arch_fmt: The archive format used in the test
        '''
        self._set_artifact_paths(arch_fmt)

        # Remove the artifacts if any present
        if any([os.path.exists(f) for f in (self.src, self.arch, self.dst)]):
            self._tear_down()
            self._set_artifact_paths(arch_fmt)

        # Create source
        os.makedirs(self.src)
        if unicode_filename:
            filename = 'file®'
        else:
            filename = 'file'
        with salt.utils.files.fopen(os.path.join(self.src, filename), 'wb') as theorem:
            if six.PY3 and salt.utils.platform.is_windows():
                encoding = 'utf-8'
            else:
                encoding = None
            theorem.write(salt.utils.stringutils.to_bytes(textwrap.dedent('''\
                Compression theorem of computational complexity theory:

                Given a Gödel numbering $φ$ of the computable functions and a
                Blum complexity measure $Φ$ where a complexity class for a
                boundary function $f$ is defined as

                    $\\mathrm C(f) := \\{φ_i ∈ \\mathbb R^{(1)} | (∀^∞ x) Φ_i(x) ≤ f(x)\\}$.

                Then there exists a total computable function $f$ so that for
                all $i$

                    $\\mathrm{Dom}(φ_i) = \\mathrm{Dom}(φ_{f(i)})$

                and

                    $\\mathrm C(φ_i) ⊊ \\mathrm{C}(φ_{f(i)})$.
            '''), encoding=encoding))

        # Create destination
        os.makedirs(self.dst)

    def _tear_down(self):
        '''
        Remove source file tree, archive, and destination file tree
        '''
        for f in (self.src, self.arch, self.dst):
            if os.path.exists(f):
                if os.path.isdir(f):
                    shutil.rmtree(f, ignore_errors=True)
                else:
                    os.remove(f)
        del self.dst
        del self.src
        del self.arch
        del self.src_file

    def _assert_artifacts_in_ret(self, ret, file_only=False, unix_sep=False):
        '''
        Assert that the artifact source files are printed in the source command
        output
        '''

        def normdir(path):
            normdir = os.path.normcase(os.path.abspath(path))
            if salt.utils.platform.is_windows():
                # Remove the drive portion of path
                if len(normdir) >= 2 and normdir[1] == ':':
                    normdir = normdir.split(':', 1)[1]
            normdir = normdir.lstrip(os.path.sep)
            # Unzipped paths might have unix line endings
            if unix_sep:
                normdir = normdir.replace(os.path.sep, '/')
            return normdir

        # Try to find source directory and file in output lines
        dir_in_ret = None
        file_in_ret = None
        for line in ret:
            if normdir(self.src) in os.path.normcase(line) \
            and not normdir(self.src_file) in os.path.normcase(line):
                dir_in_ret = True
            if normdir(self.src_file) in os.path.normcase(line):
                file_in_ret = True

        # Assert number of lines, reporting of source directory and file
        self.assertTrue(len(ret) >= 1 if file_only else 2)
        if not file_only:
            self.assertTrue(dir_in_ret)
        self.assertTrue(file_in_ret)

    @skipIf(not salt.utils.path.which('tar'), 'Cannot find tar executable')
    def test_tar_pack(self):
        '''
        Validate using the tar function to create archives
        '''
        self._set_up(arch_fmt='tar')

        # Test create archive
        ret = self.run_function('archive.tar', ['-cvf', self.arch], sources=self.src)
        self.assertTrue(isinstance(ret, list), six.text_type(ret))
        self._assert_artifacts_in_ret(ret)

        self._tear_down()

    @skipIf(not salt.utils.path.which('tar'), 'Cannot find tar executable')
    def test_tar_unpack(self):
        '''
        Validate using the tar function to extract archives
        '''
        self._set_up(arch_fmt='tar')
        self.run_function('archive.tar', ['-cvf', self.arch], sources=self.src)

        # Test extract archive
        ret = self.run_function('archive.tar', ['-xvf', self.arch], dest=self.dst)
        self.assertTrue(isinstance(ret, list), six.text_type(ret))
        self._assert_artifacts_in_ret(ret)

        self._tear_down()

    @skipIf(not salt.utils.path.which('tar'), 'Cannot find tar executable')
    def test_tar_pack_unicode(self):
        '''
        Validate using the tar function to create archives
        '''
        self._set_up(arch_fmt='tar', unicode_filename=True)

        # Test create archive
        ret = self.run_function('archive.tar', ['-cvf', self.arch], sources=self.src)
        self.assertTrue(isinstance(ret, list), six.text_type(ret))
        self._assert_artifacts_in_ret(ret)

        self._tear_down()

    @skipIf(not salt.utils.path.which('tar'), 'Cannot find tar executable')
    def test_tar_unpack_unicode(self):
        '''
        Validate using the tar function to extract archives
        '''
        self._set_up(arch_fmt='tar', unicode_filename=True)
        self.run_function('archive.tar', ['-cvf', self.arch], sources=self.src)

        # Test extract archive
        ret = self.run_function('archive.tar', ['-xvf', self.arch], dest=self.dst)
        self.assertTrue(isinstance(ret, list), six.text_type(ret))
        self._assert_artifacts_in_ret(ret)

        self._tear_down()

    @skipIf(not salt.utils.path.which('tar'), 'Cannot find tar executable')
    def test_tar_list_unicode(self):
        '''
        Validate using the tar function to extract archives
        '''
        self._set_up(arch_fmt='tar', unicode_filename=True)
        self.run_function('archive.tar', ['-cvf', self.arch], sources=self.src)

        # Test list archive
        ret = self.run_function('archive.list', name=self.arch)
        self.assertTrue(isinstance(ret, list), six.text_type(ret))
        self._assert_artifacts_in_ret(ret)

        self._tear_down()

    @skipIf(not salt.utils.path.which('gzip'), 'Cannot find gzip executable')
    def test_gzip(self):
        '''
        Validate using the gzip function
        '''
        self._set_up(arch_fmt='gz')

        # Test create archive
        ret = self.run_function('archive.gzip', [self.src_file], options='-v')
        self.assertTrue(isinstance(ret, list), six.text_type(ret))
        self._assert_artifacts_in_ret(ret, file_only=True)

        self._tear_down()

    @skipIf(not salt.utils.path.which('gzip'), 'Cannot find gzip executable')
    @skipIf(not salt.utils.path.which('gunzip'), 'Cannot find gunzip executable')
    def test_gunzip(self):
        '''
        Validate using the gunzip function
        '''
        self._set_up(arch_fmt='gz')
        self.run_function('archive.gzip', [self.src_file], options='-v')

        # Test extract archive
        ret = self.run_function('archive.gunzip', [self.src_file + '.gz'], options='-v')
        self.assertTrue(isinstance(ret, list), six.text_type(ret))
        self._assert_artifacts_in_ret(ret, file_only=True)

        self._tear_down()

    @skipIf(not salt.utils.path.which('zip'), 'Cannot find zip executable')
    def test_cmd_zip(self):
        '''
        Validate using the cmd_zip function
        '''
        self._set_up(arch_fmt='zip')

        # Test create archive
        ret = self.run_function('archive.cmd_zip', [self.arch, self.src])
        self.assertTrue(isinstance(ret, list), six.text_type(ret))
        self._assert_artifacts_in_ret(ret)

        self._tear_down()

    @skipIf(not salt.utils.path.which('zip'), 'Cannot find zip executable')
    @skipIf(not salt.utils.path.which('unzip'), 'Cannot find unzip executable')
    def test_cmd_unzip(self):
        '''
        Validate using the cmd_unzip function
        '''
        self._set_up(arch_fmt='zip')
        self.run_function('archive.cmd_zip', [self.arch, self.src])

        # Test create archive
        ret = self.run_function('archive.cmd_unzip', [self.arch, self.dst])
        self.assertTrue(isinstance(ret, list), six.text_type(ret))
        self._assert_artifacts_in_ret(ret)

        self._tear_down()

    @skipIf(not HAS_ZIPFILE, 'Cannot find zipfile python module')
    def test_zip(self):
        '''
        Validate using the zip function
        '''
        self._set_up(arch_fmt='zip')

        # Test create archive
        ret = self.run_function('archive.zip', [self.arch, self.src])
        self.assertTrue(isinstance(ret, list), six.text_type(ret))
        self._assert_artifacts_in_ret(ret)

        self._tear_down()

    @skipIf(not HAS_ZIPFILE, 'Cannot find zipfile python module')
    def test_unzip(self):
        '''
        Validate using the unzip function
        '''
        self._set_up(arch_fmt='zip')
        self.run_function('archive.zip', [self.arch, self.src])

        # Test create archive
        ret = self.run_function('archive.unzip', [self.arch, self.dst])
        self.assertTrue(isinstance(ret, list), six.text_type(ret))
        self._assert_artifacts_in_ret(ret, unix_sep=False)

        self._tear_down()

    @skipIf(not salt.utils.path.which('rar'), 'Cannot find rar executable')
    def test_rar(self):
        '''
        Validate using the rar function
        '''
        self._set_up(arch_fmt='rar')

        # Test create archive
        ret = self.run_function('archive.rar', [self.arch, self.src])
        self.assertTrue(isinstance(ret, list), six.text_type(ret))
        self._assert_artifacts_in_ret(ret)

        self._tear_down()

    @skipIf(not salt.utils.path.which('rar'), 'Cannot find rar executable')
    @skipIf(not salt.utils.path.which('unrar'), 'Cannot find unrar executable')
    def test_unrar(self):
        '''
        Validate using the unrar function
        '''
        self._set_up(arch_fmt='rar')
        self.run_function('archive.rar', [self.arch, self.src])

        # Test create archive
        ret = self.run_function('archive.unrar', [self.arch, self.dst])
        self.assertTrue(isinstance(ret, list), six.text_type(ret))
        self._assert_artifacts_in_ret(ret)

        self._tear_down()



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function
import os

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.win_path as win_path
import salt.utils.stringutils


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinPathTestCase(TestCase, LoaderModuleMockMixin):
    '''
        Test cases for salt.modules.win_path
    '''
    def setup_loader_modules(self):
        return {win_path: {}}

    def __init__(self, *args, **kwargs):
        super(WinPathTestCase, self).__init__(*args, **kwargs)
        self.pathsep = str(';')  # future lint: disable=blacklisted-function

    def assert_call_matches(self, mock_obj, new_path):
        mock_obj.assert_called_once_with(
            win_path.HIVE,
            win_path.KEY,
            win_path.VNAME,
            self.pathsep.join(new_path),
            win_path.VTYPE
        )

    def assert_path_matches(self, env, new_path):
        self.assertEqual(
            env['PATH'],
            salt.utils.stringutils.to_str(self.pathsep.join(new_path))
        )

    def test_get_path(self):
        '''
        Test to Returns the system path
        '''
        mock = MagicMock(return_value={'vdata': 'C:\\Salt'})
        with patch.dict(win_path.__salt__, {'reg.read_value': mock}):
            self.assertListEqual(win_path.get_path(), ['C:\\Salt'])

    def test_exists(self):
        '''
        Test to check if the directory is configured
        '''
        get_mock = MagicMock(return_value=['C:\\Foo', 'C:\\Bar'])
        with patch.object(win_path, 'get_path', get_mock):
            # Ensure case insensitivity respected
            self.assertTrue(win_path.exists('C:\\FOO'))
            self.assertTrue(win_path.exists('c:\\foo'))
            self.assertFalse(win_path.exists('c:\\mystuff'))

    def test_add(self):
        '''
        Test to add the directory to the SYSTEM path
        '''
        orig_path = ('C:\\Foo', 'C:\\Bar')

        def _env(path):
            return {
                str('PATH'): salt.utils.stringutils.to_str(  # future lint: disable=blacklisted-function
                    self.pathsep.join(path)
                )
            }

        def _run(name, index=None, retval=True, path=None):
            if path is None:
                path = orig_path
            env = _env(path)
            mock_get = MagicMock(return_value=list(path))
            mock_set = MagicMock(return_value=retval)
            with patch.object(win_path, 'PATHSEP', self.pathsep), \
                    patch.object(win_path, 'get_path', mock_get), \
                    patch.object(os, 'environ', env), \
                    patch.dict(win_path.__salt__, {'reg.set_value': mock_set}), \
                    patch.object(win_path, 'rehash', MagicMock(return_value=True)):
                return win_path.add(name, index), env, mock_set

        # Test a successful reg update
        ret, env, mock_set = _run('c:\\salt', retval=True)
        new_path = ('C:\\Foo', 'C:\\Bar', 'c:\\salt')
        self.assertTrue(ret)
        self.assert_call_matches(mock_set, new_path)
        self.assert_path_matches(env, new_path)

        # Test an unsuccessful reg update
        ret, env, mock_set = _run('c:\\salt', retval=False)
        new_path = ('C:\\Foo', 'C:\\Bar', 'c:\\salt')
        self.assertFalse(ret)
        self.assert_call_matches(mock_set, new_path)
        # The local path should still have been modified even
        # though reg.set_value failed.
        self.assert_path_matches(env, new_path)

        # Test adding with a custom index
        ret, env, mock_set = _run('c:\\salt', index=1, retval=True)
        new_path = ('C:\\Foo', 'c:\\salt', 'C:\\Bar')
        self.assertTrue(ret)
        self.assert_call_matches(mock_set, new_path)
        self.assert_path_matches(env, new_path)

        # Test adding path with a case-insensitive match already present, and
        # no index provided. The path should remain unchanged and we should not
        # update the registry.
        ret, env, mock_set = _run('c:\\foo', retval=True)
        self.assertTrue(ret)
        mock_set.assert_not_called()
        self.assert_path_matches(env, orig_path)

        # Test adding path with a case-insensitive match already present, and a
        # negative index provided which does not match the current index. The
        # match should be removed, and the path should be added to the end of
        # the list.
        ret, env, mock_set = _run('c:\\foo', index=-1, retval=True)
        new_path = ('C:\\Bar', 'c:\\foo')
        self.assertTrue(ret)
        self.assert_call_matches(mock_set, new_path)
        self.assert_path_matches(env, new_path)

        # Test adding path with a case-insensitive match already present, and a
        # negative index provided which matches the current index. No changes
        # should be made.
        ret, env, mock_set = _run('c:\\foo', index=-2, retval=True)
        self.assertTrue(ret)
        mock_set.assert_not_called()
        self.assert_path_matches(env, orig_path)

        # Test adding path with a case-insensitive match already present, and a
        # negative index provided which is larger than the size of the list. No
        # changes should be made, since in these cases we assume an index of 0,
        # and the case-insensitive match is also at index 0.
        ret, env, mock_set = _run('c:\\foo', index=-5, retval=True)
        self.assertTrue(ret)
        mock_set.assert_not_called()
        self.assert_path_matches(env, orig_path)

        # Test adding path with a case-insensitive match already present, and a
        # negative index provided which is larger than the size of the list.
        # The match should be removed from its current location and inserted at
        # the beginning, since when a negative index is larger than the list,
        # we put it at the beginning of the list.
        ret, env, mock_set = _run('c:\\bar', index=-5, retval=True)
        new_path = ('c:\\bar', 'C:\\Foo')
        self.assertTrue(ret)
        self.assert_call_matches(mock_set, new_path)
        self.assert_path_matches(env, new_path)

        # Test adding path with a case-insensitive match already present, and a
        # negative index provided which matches the current index. The path
        # should remain unchanged and we should not update the registry.
        ret, env, mock_set = _run('c:\\bar', index=-1, retval=True)
        self.assertTrue(ret)
        mock_set.assert_not_called()
        self.assert_path_matches(env, orig_path)

        # Test adding path with a case-insensitive match already present, and
        # an index provided which does not match the current index, and is also
        # larger than the size of the PATH list. The match should be removed,
        # and the path should be added to the end of the list.
        ret, env, mock_set = _run('c:\\foo', index=5, retval=True)
        new_path = ('C:\\Bar', 'c:\\foo')
        self.assertTrue(ret)
        self.assert_call_matches(mock_set, new_path)
        self.assert_path_matches(env, new_path)

    def test_remove(self):
        '''
        Test win_path.remove
        '''
        orig_path = ('C:\\Foo', 'C:\\Bar', 'C:\\Baz')

        def _env(path):
            return {
                str('PATH'): salt.utils.stringutils.to_str(  # future lint: disable=blacklisted-function
                    self.pathsep.join(path)
                )
            }

        def _run(name='c:\\salt', index=None, retval=True, path=None):
            if path is None:
                path = orig_path
            env = _env(path)
            mock_get = MagicMock(return_value=list(path))
            mock_set = MagicMock(return_value=retval)
            with patch.object(win_path, 'PATHSEP', self.pathsep), \
                    patch.object(win_path, 'get_path', mock_get), \
                    patch.object(os, 'environ', env), \
                    patch.dict(win_path.__salt__, {'reg.set_value': mock_set}), \
                    patch.object(win_path, 'rehash', MagicMock(return_value=True)):
                return win_path.remove(name), env, mock_set

        # Test a successful reg update
        ret, env, mock_set = _run('C:\\Bar', retval=True)
        new_path = ('C:\\Foo', 'C:\\Baz')
        self.assertTrue(ret)
        self.assert_call_matches(mock_set, new_path)
        self.assert_path_matches(env, new_path)

        # Test a successful reg update with a case-insensitive match
        ret, env, mock_set = _run('c:\\bar', retval=True)
        new_path = ('C:\\Foo', 'C:\\Baz')
        self.assertTrue(ret)
        self.assert_call_matches(mock_set, new_path)
        self.assert_path_matches(env, new_path)

        # Test a successful reg update with multiple case-insensitive matches.
        # All matches should be removed.
        old_path = orig_path + ('C:\\BAR',)
        ret, env, mock_set = _run('c:\\bar', retval=True)
        new_path = ('C:\\Foo', 'C:\\Baz')
        self.assertTrue(ret)
        self.assert_call_matches(mock_set, new_path)
        self.assert_path_matches(env, new_path)

        # Test an unsuccessful reg update
        ret, env, mock_set = _run('c:\\bar', retval=False)
        new_path = ('C:\\Foo', 'C:\\Baz')
        self.assertFalse(ret)
        self.assert_call_matches(mock_set, new_path)
        # The local path should still have been modified even
        # though reg.set_value failed.
        self.assert_path_matches(env, new_path)

        # Test when no match found
        ret, env, mock_set = _run('C:\\NotThere', retval=True)
        self.assertTrue(ret)
        mock_set.assert_not_called()
        self.assert_path_matches(env, orig_path)
